@@ -1,10 +1,14 @@
 import numpy as np
 from skimage.transform import SimilarityTransform, warp
-from skimage.util.dtype import _convert as convert
 import warnings
 import functools
 import inspect
 
+try:
+    from skimage.util.dtype import _convert as convert
+except ImportError:
+    from skimage.util.dtype import convert
+
 
 class WarpSuite:
     params = ([np.uint8, np.uint16, np.float32, np.float64],
