@@ -4,7 +4,6 @@ Specific images
 ===============
 
 """
-import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib
 
@@ -28,6 +27,24 @@ ax[1].imshow(images[1])
 fig.tight_layout()
 plt.show()
 
+
+######################################################################
+#
+# PIV images
+# =============
+
+
+fig, axes = plt.subplots(1, 2, figsize=(8, 4))
+ax = axes.ravel()
+
+images = data.vortex()
+ax[0].imshow(images[0])
+ax[1].imshow(images[1])
+
+fig.tight_layout()
+plt.show()
+
+
 ######################################################################
 #
 # Faces and non-faces dataset
