@@ -13,7 +13,7 @@ from skimage.draw import random_shapes
 # Let's start simple and generate a 128x128 image
 # with a single grayscale rectangle.
 result = random_shapes((128, 128), max_shapes=1, shape='rectangle',
-                       multichannel=False)
+                       multichannel=False, random_seed=0)
 
 # We get back a tuple consisting of (1) the image with the generated shapes
 # and (2) a list of label tuples with the kind of shape (e.g. circle,
