@@ -24,6 +24,6 @@ outer_disk_mask = (X - l_x / 2)**2 + (Y - l_y / 2)**2 > (l_x / 2)**2
 camera[outer_disk_mask] = 0
 
 plt.figure(figsize=(4, 4))
-plt.imshow(camera, cmap='gray', interpolation='nearest')
+plt.imshow(camera, cmap='gray')
 plt.axis('off')
 plt.show()
