@@ -20,12 +20,13 @@ from matplotlib import pyplot as plt
 
 img = data.coffee()
 
-labels1 = segmentation.slic(img, compactness=30, n_segments=400)
-out1 = color.label2rgb(labels1, img, kind='avg')
+labels1 = segmentation.slic(img, compactness=30, n_segments=400,
+                            start_label=1)
+out1 = color.label2rgb(labels1, img, kind='avg', bg_label=0)
 
 g = graph.rag_mean_color(img, labels1, mode='similarity')
 labels2 = graph.cut_normalized(labels1, g)
-out2 = color.label2rgb(labels2, img, kind='avg')
+out2 = color.label2rgb(labels2, img, kind='avg', bg_label=0)
 
 fig, ax = plt.subplots(nrows=2, sharex=True, sharey=True, figsize=(6, 8))
 
