@@ -80,7 +80,7 @@ more regularly shaped. [5]_
 .. [5] Peer Neubert & Peter Protzel (2014). Compact Watershed and
        Preemptive SLIC: On Improving Trade-offs of Superpixel Segmentation
        Algorithms. ICPR 2014, pp 996-1001. :DOI:`10.1109/ICPR.2014.181`
-       https://www.tu-chemnitz.de/etit/proaut/forschung/rsrc/cws_pSLIC_ICPR.pdf
+       https://www.tu-chemnitz.de/etit/proaut/publications/cws_pSLIC_ICPR.pdf
 """
 
 import matplotlib.pyplot as plt
@@ -96,7 +96,8 @@ from skimage.util import img_as_float
 img = img_as_float(astronaut()[::2, ::2])
 
 segments_fz = felzenszwalb(img, scale=100, sigma=0.5, min_size=50)
-segments_slic = slic(img, n_segments=250, compactness=10, sigma=1)
+segments_slic = slic(img, n_segments=250, compactness=10, sigma=1,
+                     start_label=1)
 segments_quick = quickshift(img, kernel_size=3, max_dist=6, ratio=0.5)
 gradient = sobel(rgb2gray(img))
 segments_watershed = watershed(gradient, markers=250, compactness=0.001)
