@@ -1,7 +1,7 @@
 import sys
 import os
-import hashlib
 from distutils.version import LooseVersion
+from multiprocessing import cpu_count
 
 CYTHON_VERSION = '0.23.4'
 
@@ -13,6 +13,19 @@ except NameError:
         pass
 
 
+def _compiled_filename(f):
+    """Check for the presence of a .pyx[.in] file as a .c or .cpp."""
+    basename = f.replace('.in', '').replace('.pyx', '')
+    for ext in ('.c', '.cpp'):
+        filename = basename + ext
+        if os.path.exists(filename):
+            return filename
+    else:
+        raise RuntimeError('Cython >= %s is required to build '
+                           'scikit-image from git checkout' %
+                           CYTHON_VERSION)
+
+
 def cython(pyx_files, working_path=''):
     """Use Cython to convert the given files to C.
 
@@ -34,58 +47,23 @@ def cython(pyx_files, working_path=''):
         from Cython.Build import cythonize
     except ImportError:
         # If cython is not found, the build will make use of
-        # the distributed .c files if present
-        c_files = [f.replace('.pyx.in', '.c').replace('.pyx', '.c') for f in pyx_files]
-        for cfile in [os.path.join(working_path, f) for f in c_files]:
-            if not os.path.isfile(cfile):
-                raise RuntimeError('Cython >= %s is required to build scikit-image from git checkout' \
-                                   % CYTHON_VERSION)
+        # the distributed .c or .cpp files if present
+        c_files_used = [_compiled_filename(os.path.join(working_path, f))
+                        for f in pyx_files]
 
         print("Cython >= %s not found; falling back to pre-built %s" \
-              % (CYTHON_VERSION, " ".join(c_files)))
+              % (CYTHON_VERSION, " ".join(c_files_used)))
     else:
-        for pyxfile in [os.path.join(working_path, f) for f in pyx_files]:
-
-            # if the .pyx file stayed the same, we don't need to recompile
-            if not _changed(pyxfile):
-                continue
-
+        pyx_files = [os.path.join(working_path, f) for f in pyx_files]
+        for i, pyxfile in enumerate(pyx_files):
             if pyxfile.endswith('.pyx.in'):
                 process_tempita_pyx(pyxfile)
-                pyxfile = pyxfile.replace('.pyx.in', '.pyx')
+                pyx_files[i] = pyxfile.replace('.pyx.in', '.pyx')
 
-            cythonize(pyxfile)
-
-
-def _md5sum(f):
-    m = hashlib.new('md5')
-    while True:
-        # Hash one 8096 byte block at a time
-        d = f.read(8096)
-        if not d:
-            break
-        m.update(d)
-    return m.hexdigest()
-
-
-def _changed(filename):
-    """Compare the hash of a Cython file to the cached hash value on disk.
-
-    """
-    filename_cache = filename + '.md5'
-
-    try:
-        md5_cached = open(filename_cache, 'rb').read()
-    except IOError:
-        md5_cached = '0'
-
-    with open(filename, 'rb') as f:
-        md5_new = _md5sum(f)
-
-        with open(filename_cache, 'wb') as cf:
-            cf.write(md5_new.encode('utf-8'))
-
-    return md5_cached != md5_new.encode('utf-8')
+        # Cython doesn't automatically choose a number of threads > 1
+        # https://github.com/cython/cython/blob/a0bbb940c847dfe92cac446c8784c34c28c92836/Cython/Build/Dependencies.py#L923-L925
+        cythonize(pyx_files, nthreads=cpu_count(),
+                  compiler_directives={'language_level': 3})
 
 
 def process_tempita_pyx(fromfile):
