@@ -8,15 +8,13 @@ Authors
 :license: modified BSD
 """
 
-import os.path
-
 import numpy as np
+import pytest
 from skimage._shared.testing import assert_equal, assert_almost_equal
-from skimage._shared.testing import assert_array_almost_equal
+from skimage._shared.testing import assert_array_almost_equal, fetch
 from skimage._shared.testing import TestCase
 
-from skimage.util import img_as_float, img_as_ubyte
-from skimage.io import imread
+from skimage.util import img_as_float, img_as_ubyte, img_as_float32
 from skimage.color import (rgb2hsv, hsv2rgb,
                            rgb2xyz, xyz2rgb,
                            rgb2hed, hed2rgb,
@@ -24,7 +22,7 @@ from skimage.color import (rgb2hsv, hsv2rgb,
                            combine_stains,
                            rgb2rgbcie, rgbcie2rgb,
                            convert_colorspace,
-                           rgb2grey, gray2rgb,
+                           rgb2gray, gray2rgb,
                            xyz2lab, lab2xyz,
                            lab2rgb, rgb2lab,
                            xyz2luv, luv2xyz,
@@ -35,24 +33,24 @@ from skimage.color import (rgb2hsv, hsv2rgb,
                            rgb2ypbpr, ypbpr2rgb,
                            rgb2ycbcr, ycbcr2rgb,
                            rgb2ydbdr, ydbdr2rgb,
-                           rgba2rgb)
+                           rgba2rgb, gray2rgba)
 
-from skimage import data_dir
 from skimage._shared._warnings import expected_warnings
+from skimage import data
 import colorsys
 
 
 class TestColorconv(TestCase):
 
-    img_rgb = imread(os.path.join(data_dir, 'color.png'))
-    img_grayscale = imread(os.path.join(data_dir, 'camera.png'))
+    img_rgb = data.colorwheel()
+    img_grayscale = data.camera()
     img_rgba = np.array([[[0, 0.5, 1, 0],
                           [0, 0.5, 1, 1],
-                          [0, 0.5, 1, 0.5]]]).astype(np.float)
+                          [0, 0.5, 1, 0.5]]]).astype(float)
 
     colbars = np.array([[1, 1, 0, 0, 1, 1, 0, 0],
                         [1, 1, 1, 1, 0, 0, 0, 0],
-                        [1, 0, 1, 0, 1, 0, 1, 0]]).astype(np.float)
+                        [1, 0, 1, 0, 1, 0, 1, 0]]).astype(float)
     colbars_array = np.swapaxes(colbars.reshape(3, 4, 2), 0, 2)
     colbars_point75 = colbars * 0.75
     colbars_point75_array = np.swapaxes(colbars_point75.reshape(3, 4, 2), 0, 2)
@@ -83,7 +81,7 @@ class TestColorconv(TestCase):
         rgb = rgba2rgb(rgba)
         expected = np.array([[[1, 1, 1],
                               [0, 0.5, 1],
-                              [0.5, 0.75, 1]]]).astype(np.float)
+                              [0.5, 0.75, 1]]]).astype(float)
         self.assertEqual(rgb.shape, expected.shape)
         assert_almost_equal(rgb, expected)
 
@@ -93,6 +91,13 @@ class TestColorconv(TestCase):
     def test_rgba2rgb_error_rgb(self):
         self.assertRaises(ValueError, rgba2rgb, self.img_rgb)
 
+    def test_rgba2rgb_dtype(self):
+        rgba = self.img_rgba.astype('float64')
+        rgba32 = img_as_float32(rgba)
+
+        assert rgba2rgb(rgba).dtype == rgba.dtype
+        assert rgba2rgb(rgba32).dtype == rgba32.dtype
+
     # RGB to HSV
     def test_rgb2hsv_conversion(self):
         rgb = img_as_float(self.img_rgb)[::16, ::16]
@@ -106,8 +111,12 @@ class TestColorconv(TestCase):
     def test_rgb2hsv_error_grayscale(self):
         self.assertRaises(ValueError, rgb2hsv, self.img_grayscale)
 
-    def test_rgb2hsv_error_one_element(self):
-        self.assertRaises(ValueError, rgb2hsv, self.img_rgb[0, 0])
+    def test_rgb2hsv_dtype(self):
+        rgb = img_as_float(self.img_rgb)
+        rgb32 = img_as_float32(self.img_rgb)
+
+        assert rgb2hsv(rgb).dtype == rgb.dtype
+        assert rgb2hsv(rgb32).dtype == rgb32.dtype
 
     # HSV to RGB
     def test_hsv2rgb_conversion(self):
@@ -122,8 +131,16 @@ class TestColorconv(TestCase):
     def test_hsv2rgb_error_grayscale(self):
         self.assertRaises(ValueError, hsv2rgb, self.img_grayscale)
 
-    def test_hsv2rgb_error_one_element(self):
-        self.assertRaises(ValueError, hsv2rgb, self.img_rgb[0, 0])
+    def test_hsv2rgb_dtype(self):
+        rgb = self.img_rgb.astype("float32")[::16, ::16]
+        # create HSV image with colorsys
+        hsv = np.array([colorsys.rgb_to_hsv(pt[0], pt[1], pt[2])
+                        for pt in rgb.reshape(-1, 3)],
+                       dtype='float64').reshape(rgb.shape)
+        hsv32 = hsv.astype('float32')
+
+        assert hsv2rgb(hsv).dtype == hsv.dtype
+        assert hsv2rgb(hsv32).dtype == hsv32.dtype
 
     # RGB to XYZ
     def test_rgb2xyz_conversion(self):
@@ -142,14 +159,25 @@ class TestColorconv(TestCase):
     def test_rgb2xyz_error_grayscale(self):
         self.assertRaises(ValueError, rgb2xyz, self.img_grayscale)
 
-    def test_rgb2xyz_error_one_element(self):
-        self.assertRaises(ValueError, rgb2xyz, self.img_rgb[0, 0])
+    def test_rgb2xyz_dtype(self):
+        img = self.colbars_array
+        img32 = img.astype('float32')
+
+        assert rgb2xyz(img).dtype == img.dtype
+        assert rgb2xyz(img32).dtype == img32.dtype
 
     # XYZ to RGB
     def test_xyz2rgb_conversion(self):
         assert_almost_equal(xyz2rgb(rgb2xyz(self.colbars_array)),
                             self.colbars_array)
 
+    def test_xyz2rgb_dtype(self):
+        img = rgb2xyz(self.colbars_array)
+        img32 = img.astype('float32')
+
+        assert xyz2rgb(img).dtype == img.dtype
+        assert xyz2rgb(img32).dtype == img32.dtype
+
     # RGB<->XYZ roundtrip on another image
     def test_xyz_rgb_roundtrip(self):
         img_rgb = img_as_float(self.img_rgb)
@@ -194,12 +222,26 @@ class TestColorconv(TestCase):
                         [ 0.        ,  0.        ,  0.        ]]])
         assert_almost_equal(rgb2rgbcie(self.colbars_array), gt)
 
+    def test_rgb2rgbcie_dtype(self):
+        img = self.colbars_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert rgb2rgbcie(img).dtype == img.dtype
+        assert rgb2rgbcie(img32).dtype == img32.dtype
+
     # RGB CIE to RGB
     def test_rgbcie2rgb_conversion(self):
         # only roundtrip test, we checked rgb2rgbcie above already
         assert_almost_equal(rgbcie2rgb(rgb2rgbcie(self.colbars_array)),
                             self.colbars_array)
 
+    def test_rgbcie2rgb_dtype(self):
+        img = rgb2rgbcie(self.colbars_array).astype('float64')
+        img32 = img.astype('float32')
+
+        assert rgbcie2rgb(img).dtype == img.dtype
+        assert rgbcie2rgb(img32).dtype == img32.dtype
+
     def test_convert_colorspace(self):
         colspaces = ['HSV', 'RGB CIE', 'XYZ', 'YCbCr', 'YPbPr', 'YDbDr']
         colfuncs_from = [
@@ -228,24 +270,33 @@ class TestColorconv(TestCase):
         self.assertRaises(ValueError, convert_colorspace,
                           self.colbars_array, 'RGB', 'nokey')
 
-    def test_rgb2grey(self):
-        x = np.array([1, 1, 1]).reshape((1, 1, 3)).astype(np.float)
-        g = rgb2grey(x)
+    def test_rgb2gray(self):
+        x = np.array([1, 1, 1]).reshape((1, 1, 3)).astype(float)
+        g = rgb2gray(x)
         assert_array_almost_equal(g, 1)
 
         assert_equal(g.shape, (1, 1))
 
-    def test_rgb2grey_contiguous(self):
+    def test_rgb2gray_contiguous(self):
         x = np.random.rand(10, 10, 3)
-        assert rgb2grey(x).flags["C_CONTIGUOUS"]
-        assert rgb2grey(x[:5, :5]).flags["C_CONTIGUOUS"]
+        assert rgb2gray(x).flags["C_CONTIGUOUS"]
+        assert rgb2gray(x[:5, :5]).flags["C_CONTIGUOUS"]
 
-    def test_rgb2grey_alpha(self):
+    def test_rgb2gray_alpha(self):
         x = np.random.rand(10, 10, 4)
-        assert rgb2grey(x).ndim == 2
+        with expected_warnings(['Non RGB image conversion']):
+            assert rgb2gray(x).ndim == 2
 
-    def test_rgb2grey_on_grey(self):
-        rgb2grey(np.random.rand(5, 5))
+    def test_rgb2gray_on_gray(self):
+        with expected_warnings(['The behavior of rgb2gray will change']):
+            rgb2gray(np.random.rand(5, 5))
+
+    def test_rgb2gray_dtype(self):
+        img = np.random.rand(10, 10, 3).astype('float64')
+        img32 = img.astype('float32')
+
+        assert rgb2gray(img).dtype == img.dtype
+        assert rgb2gray(img32).dtype == img32.dtype
 
     # test matrices for xyz2lab and lab2xyz generated using
     # http://www.easyrgb.com/index.php?X=CALC
@@ -257,20 +308,25 @@ class TestColorconv(TestCase):
         # Test the conversion with the rest of the illuminants.
         for I in ["d50", "d55", "d65", "d75"]:
             for obs in ["2", "10"]:
-                fname = "lab_array_{0}_{1}.npy".format(I, obs)
-                lab_array_I_obs = np.load(
-                    os.path.join(os.path.dirname(__file__), 'data', fname))
+                fname = "color/tests/data/lab_array_{0}_{1}.npy".format(I, obs)
+                lab_array_I_obs = np.load(fetch(fname))
                 assert_array_almost_equal(lab_array_I_obs,
                                           xyz2lab(self.xyz_array, I, obs),
                                           decimal=2)
         for I in ["a", "e"]:
-            fname = "lab_array_{0}_2.npy".format(I)
-            lab_array_I_obs = np.load(
-                os.path.join(os.path.dirname(__file__), 'data', fname))
+            fname = "color/tests/data/lab_array_{0}_2.npy".format(I)
+            lab_array_I_obs = np.load(fetch(fname))
             assert_array_almost_equal(lab_array_I_obs,
                                       xyz2lab(self.xyz_array, I, "2"),
                                       decimal=2)
 
+    def test_xyz2lab_dtype(self):
+        img = self.xyz_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert xyz2lab(img).dtype == img.dtype
+        assert xyz2lab(img32).dtype == img32.dtype
+
     def test_lab2xyz(self):
         assert_array_almost_equal(lab2xyz(self.lab_array),
                                   self.xyz_array, decimal=3)
@@ -278,15 +334,13 @@ class TestColorconv(TestCase):
         # Test the conversion with the rest of the illuminants.
         for I in ["d50", "d55", "d65", "d75"]:
             for obs in ["2", "10"]:
-                fname = "lab_array_{0}_{1}.npy".format(I, obs)
-                lab_array_I_obs = np.load(
-                    os.path.join(os.path.dirname(__file__), 'data', fname))
+                fname = "color/tests/data/lab_array_{0}_{1}.npy".format(I, obs)
+                lab_array_I_obs = np.load(fetch(fname))
                 assert_array_almost_equal(lab2xyz(lab_array_I_obs, I, obs),
                                           self.xyz_array, decimal=3)
         for I in ["a", "e"]:
             fname = "lab_array_{0}_2.npy".format(I, obs)
-            lab_array_I_obs = np.load(
-                os.path.join(os.path.dirname(__file__), 'data', fname))
+            lab_array_I_obs = np.load(fetch('color/tests/data/' + fname))
             assert_array_almost_equal(lab2xyz(lab_array_I_obs, I, "2"),
                                       self.xyz_array, decimal=3)
 
@@ -301,6 +355,13 @@ class TestColorconv(TestCase):
         except ValueError:
             pass
 
+    def test_lab2xyz_dtype(self):
+        img = self.lab_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert lab2xyz(img).dtype == img.dtype
+        assert lab2xyz(img32).dtype == img32.dtype
+
     def test_rgb2lab_brucelindbloom(self):
         """
         Test the RGB->Lab conversion by comparing to the calculator on the
@@ -324,6 +385,20 @@ class TestColorconv(TestCase):
         img_rgb = img_as_float(self.img_rgb)
         assert_array_almost_equal(lab2rgb(rgb2lab(img_rgb)), img_rgb)
 
+    def test_rgb2lab_dtype(self):
+        img = self.colbars_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert rgb2lab(img).dtype == img.dtype
+        assert rgb2lab(img32).dtype == img32.dtype
+
+    def test_lab2rgb_dtype(self):
+        img = self.lab_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert lab2rgb(img).dtype == img.dtype
+        assert lab2rgb(img32).dtype == img32.dtype
+
     # test matrices for xyz2luv and luv2xyz generated using
     # http://www.easyrgb.com/index.php?X=CALC
     # Note: easyrgb website displays xyz*100
@@ -334,20 +409,25 @@ class TestColorconv(TestCase):
         # Test the conversion with the rest of the illuminants.
         for I in ["d50", "d55", "d65", "d75"]:
             for obs in ["2", "10"]:
-                fname = "luv_array_{0}_{1}.npy".format(I, obs)
-                luv_array_I_obs = np.load(
-                    os.path.join(os.path.dirname(__file__), 'data', fname))
+                fname = "color/tests/data/luv_array_{0}_{1}.npy".format(I, obs)
+                luv_array_I_obs = np.load(fetch(fname))
                 assert_array_almost_equal(luv_array_I_obs,
                                           xyz2luv(self.xyz_array, I, obs),
                                           decimal=2)
         for I in ["a", "e"]:
-            fname = "luv_array_{0}_2.npy".format(I)
-            luv_array_I_obs = np.load(
-                os.path.join(os.path.dirname(__file__), 'data', fname))
+            fname = "color/tests/data/luv_array_{0}_2.npy".format(I)
+            luv_array_I_obs = np.load(fetch(fname))
             assert_array_almost_equal(luv_array_I_obs,
                                       xyz2luv(self.xyz_array, I, "2"),
                                       decimal=2)
 
+    def test_xyz2luv_dtype(self):
+        img = self.xyz_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert xyz2luv(img).dtype == img.dtype
+        assert xyz2luv(img32).dtype == img32.dtype
+
     def test_luv2xyz(self):
         assert_array_almost_equal(luv2xyz(self.luv_array),
                                   self.xyz_array, decimal=3)
@@ -355,18 +435,23 @@ class TestColorconv(TestCase):
         # Test the conversion with the rest of the illuminants.
         for I in ["d50", "d55", "d65", "d75"]:
             for obs in ["2", "10"]:
-                fname = "luv_array_{0}_{1}.npy".format(I, obs)
-                luv_array_I_obs = np.load(
-                    os.path.join(os.path.dirname(__file__), 'data', fname))
+                fname = "color/tests/data/luv_array_{0}_{1}.npy".format(I, obs)
+                luv_array_I_obs = np.load(fetch(fname))
                 assert_array_almost_equal(luv2xyz(luv_array_I_obs, I, obs),
                                           self.xyz_array, decimal=3)
         for I in ["a", "e"]:
-            fname = "luv_array_{0}_2.npy".format(I, obs)
-            luv_array_I_obs = np.load(
-                os.path.join(os.path.dirname(__file__), 'data', fname))
+            fname = "color/tests/data/luv_array_{0}_2.npy".format(I, obs)
+            luv_array_I_obs = np.load(fetch(fname))
             assert_array_almost_equal(luv2xyz(luv_array_I_obs, I, "2"),
                                       self.xyz_array, decimal=3)
 
+    def test_luv2xyz_dtype(self):
+        img = self.luv_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert luv2xyz(img).dtype == img.dtype
+        assert luv2xyz(img32).dtype == img32.dtype
+
     def test_rgb2luv_brucelindbloom(self):
         """
         Test the RGB->Lab conversion by comparing to the calculator on the
@@ -387,6 +472,20 @@ class TestColorconv(TestCase):
         assert_array_almost_equal(rgb2luv(self.colbars_array),
                                   gt_array, decimal=2)
 
+    def test_rgb2luv_dtype(self):
+        img = self.colbars_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert rgb2luv(img).dtype == img.dtype
+        assert rgb2luv(img32).dtype == img32.dtype
+
+    def test_luv2rgb_dtype(self):
+        img = self.luv_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert luv2rgb(img).dtype == img.dtype
+        assert luv2rgb(img32).dtype == img32.dtype
+
     def test_luv_rgb_roundtrip(self):
         img_rgb = img_as_float(self.img_rgb)
         assert_array_almost_equal(luv2rgb(rgb2luv(img_rgb)), img_rgb)
@@ -469,6 +568,20 @@ class TestColorconv(TestCase):
         assert_array_almost_equal(ycbcr2rgb(rgb2ycbcr(img_rgb)), img_rgb)
         assert_array_almost_equal(ydbdr2rgb(rgb2ydbdr(img_rgb)), img_rgb)
 
+    def test_rgb2yuv_dtype(self):
+        img = self.colbars_array.astype('float64')
+        img32 = img.astype('float32')
+
+        assert rgb2yuv(img).dtype == img.dtype
+        assert rgb2yuv(img32).dtype == img32.dtype
+
+    def test_yuv2rgb_dtype(self):
+        img = rgb2yuv(self.colbars_array).astype('float64')
+        img32 = img.astype('float32')
+
+        assert yuv2rgb(img).dtype == img.dtype
+        assert yuv2rgb(img32).dtype == img32.dtype
+
     def test_rgb2yiq_conversion(self):
         rgb = img_as_float(self.img_rgb)[::16, ::16]
         yiq = rgb2yiq(rgb).reshape(-1, 3)
@@ -505,22 +618,160 @@ def test_gray2rgb():
 
 def test_gray2rgb_rgb():
     x = np.random.rand(5, 5, 4)
-    y = gray2rgb(x)
+    with expected_warnings(['Pass-through of possibly RGB images']):
+        y = gray2rgb(x)
     assert_equal(x, y)
 
 
 def test_gray2rgb_alpha():
     x = np.random.random((5, 5, 4))
-    assert_equal(gray2rgb(x, alpha=None).shape, (5, 5, 4))
-    assert_equal(gray2rgb(x, alpha=False).shape, (5, 5, 3))
-    assert_equal(gray2rgb(x, alpha=True).shape, (5, 5, 4))
+    with expected_warnings(['Pass-through of possibly RGB images']):
+        assert_equal(gray2rgb(x, alpha=None).shape, (5, 5, 4))
+    with expected_warnings(['Pass-through of possibly RGB images',
+                            'alpha argument is deprecated']):
+        assert_equal(gray2rgb(x, alpha=False).shape, (5, 5, 3))
+    with expected_warnings(['Pass-through of possibly RGB images',
+                            'alpha argument is deprecated']):
+        assert_equal(gray2rgb(x, alpha=True).shape, (5, 5, 4))
 
     x = np.random.random((5, 5, 3))
-    assert_equal(gray2rgb(x, alpha=None).shape, (5, 5, 3))
-    assert_equal(gray2rgb(x, alpha=False).shape, (5, 5, 3))
-    assert_equal(gray2rgb(x, alpha=True).shape, (5, 5, 4))
+    with expected_warnings(['Pass-through of possibly RGB images']):
+        assert_equal(gray2rgb(x, alpha=None).shape, (5, 5, 3))
+    with expected_warnings(['Pass-through of possibly RGB images',
+                            'alpha argument is deprecated']):
+        assert_equal(gray2rgb(x, alpha=False).shape, (5, 5, 3))
+    with expected_warnings(['Pass-through of possibly RGB images',
+                            'alpha argument is deprecated']):
+        assert_equal(gray2rgb(x, alpha=True).shape, (5, 5, 4))
 
-    assert_equal(gray2rgb(np.array([[1, 2], [3, 4.]]),
-                          alpha=True)[0, 0, 3], 1)
-    assert_equal(gray2rgb(np.array([[1, 2], [3, 4]], dtype=np.uint8),
-                          alpha=True)[0, 0, 3], 255)
+    with expected_warnings(['alpha argument is deprecated']):
+        assert_equal(gray2rgb(np.array([[1, 2], [3, 4.]]),
+                              alpha=True)[0, 0, 3], 1)
+    with expected_warnings(['alpha argument is deprecated']):
+        assert_equal(gray2rgb(np.array([[1, 2], [3, 4]], dtype=np.uint8),
+                              alpha=True)[0, 0, 3], 255)
+
+
+@pytest.mark.parametrize("shape", [(5, 5), (5, 5, 4), (5, 4, 5, 4)])
+def test_gray2rgba(shape):
+    # nD case
+    img = np.random.random(shape)
+    rgba = gray2rgba(img)
+
+    # Shape check
+    assert_equal(rgba.shape, shape + (4, ))
+
+    # dtype check
+    assert rgba.dtype == img.dtype
+
+    # RGB channels check
+    for channel in range(3):
+        assert_equal(rgba[..., channel], img)
+
+    # Alpha channel check
+    assert_equal(rgba[..., 3], 1.0)
+
+
+def test_gray2rgba_dtype():
+    img_f64 = np.random.random((5, 5))
+    img_f32 = img_f64.astype('float32')
+    img_u8 = img_as_ubyte(img_f64)
+    img_int = img_u8.astype(int)
+
+    for img in [img_f64, img_f32, img_u8, img_int]:
+        assert gray2rgba(img).dtype == img.dtype
+
+
+def test_gray2rgba_alpha():
+    img = np.random.random((5, 5))
+    img_u8 = img_as_ubyte(img)
+
+    # Default
+    alpha = None
+    rgba = gray2rgba(img, alpha)
+
+    assert_equal(rgba[..., :3], gray2rgb(img))
+    assert_equal(rgba[..., 3], 1.0)
+
+    # Scalar
+    alpha = 0.5
+    rgba = gray2rgba(img, alpha)
+
+    assert_equal(rgba[..., :3], gray2rgb(img))
+    assert_equal(rgba[..., 3], alpha)
+
+    # Array
+    alpha = np.random.random((5, 5))
+    rgba = gray2rgba(img, alpha)
+
+    assert_equal(rgba[..., :3], gray2rgb(img))
+    assert_equal(rgba[..., 3], alpha)
+
+    # Warning about alpha cast
+    alpha = 0.5
+    with expected_warnings(["alpha can't be safely cast to image dtype"]):
+        rgba = gray2rgba(img_u8, alpha)
+        assert_equal(rgba[..., :3], gray2rgb(img_u8))
+
+    # Invalid shape
+    alpha = np.random.random((5, 5, 1))
+    expected_err_msg = ("could not broadcast input array from shape (5,5,1) "
+                        "into shape (5,5)")
+
+    with pytest.raises(ValueError) as err:
+        rgba = gray2rgba(img, alpha)
+    assert expected_err_msg == str(err.value)
+
+
+@pytest.mark.parametrize("func", [rgb2gray, gray2rgb, gray2rgba])
+@pytest.mark.parametrize("shape", ([(3, ), (2, 3), (4, 5, 3), (5, 4, 5, 3),
+                                    (4, 5, 4, 5, 3)]))
+def test_nD_gray_conversion(func, shape):
+    img = np.random.rand(*shape)
+
+    msg_list = []
+    if img.ndim == 3 and func == gray2rgb:
+        msg_list.append('Pass-through of possibly RGB images in gray2rgb')
+    elif img.ndim == 2 and func == rgb2gray:
+        msg_list.append('The behavior of rgb2gray will change')
+
+    with expected_warnings(msg_list):
+        out = func(img)
+
+    common_ndim = min(out.ndim, len(shape))
+
+    assert out.shape[:common_ndim] == shape[:common_ndim]
+
+
+@pytest.mark.parametrize("func", [rgb2hsv, hsv2rgb,
+                                  rgb2xyz, xyz2rgb,
+                                  rgb2hed, hed2rgb,
+                                  rgb2rgbcie, rgbcie2rgb,
+                                  xyz2lab, lab2xyz,
+                                  lab2rgb, rgb2lab,
+                                  xyz2luv, luv2xyz,
+                                  luv2rgb, rgb2luv,
+                                  lab2lch, lch2lab,
+                                  rgb2yuv, yuv2rgb,
+                                  rgb2yiq, yiq2rgb,
+                                  rgb2ypbpr, ypbpr2rgb,
+                                  rgb2ycbcr, ycbcr2rgb,
+                                  rgb2ydbdr, ydbdr2rgb])
+@pytest.mark.parametrize("shape", ([(3, ), (2, 3), (4, 5, 3), (5, 4, 5, 3),
+                                    (4, 5, 4, 5, 3)]))
+def test_nD_color_conversion(func, shape):
+    img = np.random.rand(*shape)
+    out = func(img)
+
+    assert out.shape == img.shape
+
+
+@pytest.mark.parametrize("shape", ([(4, ), (2, 4), (4, 5, 4), (5, 4, 5, 4),
+                                    (4, 5, 4, 5, 4)]))
+def test_rgba2rgb_nD(shape):
+    img = np.random.rand(*shape)
+    out = rgba2rgb(img)
+
+    expected_shape = shape[:-1] + (3, )
+
+    assert out.shape == expected_shape
