@@ -1,10 +1,12 @@
 import numpy as np
+from skimage._shared._warnings import expected_warnings
 from skimage._shared.testing import test_parallel
 from skimage._shared import testing
 from skimage._shared.testing import assert_array_equal, assert_equal
 from skimage._shared.testing import assert_almost_equal
 
 from skimage.draw import (set_color, line, line_aa, polygon, polygon_perimeter,
+                          disk,
                           circle, circle_perimeter, circle_perimeter_aa,
                           ellipse, ellipse_perimeter,
                           _bezier_segment, bezier_curve, rectangle,
@@ -143,12 +145,13 @@ def test_line_equal_aliasing_horizontally_vertically():
 
 def test_polygon_rectangle():
     img = np.zeros((10, 10), 'uint8')
+    poly = np.array(((1, 1), (4, 1), (4, 4), (1, 4), (1, 1)))
 
-    rr, cc = polygon((1, 4, 4, 1, 1), (1, 1, 4, 4, 1))
+    rr, cc = polygon(poly[:, 0], poly[:, 1])
     img[rr, cc] = 1
 
-    img_ = np.zeros((10, 10))
-    img_[1:4, 1:4] = 1
+    img_ = np.zeros((10, 10), 'uint8')
+    img_[1:5, 1:5] = 1
 
     assert_array_equal(img, img_)
 
@@ -161,16 +164,16 @@ def test_polygon_rectangle_angular():
     img[rr, cc] = 1
 
     img_ = np.array(
-        [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-         [0, 0, 1, 1, 0, 0, 0, 0, 0, 0],
-         [0, 1, 1, 1, 1, 0, 0, 0, 0, 0],
-         [1, 1, 1, 1, 1, 1, 0, 0, 0, 0],
-         [0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
-         [0, 0, 1, 1, 1, 1, 0, 0, 0, 0],
-         [0, 0, 0, 1, 1, 0, 0, 0, 0, 0],
+        [[0, 0, 0, 1, 0, 0, 0, 0, 0, 0],
+         [0, 0, 1, 1, 1, 0, 0, 0, 0, 0],
+         [0, 1, 1, 1, 1, 1, 0, 0, 0, 0],
+         [1, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+         [0, 1, 1, 1, 1, 1, 1, 1, 0, 0],
+         [0, 0, 1, 1, 1, 1, 1, 0, 0, 0],
+         [0, 0, 0, 1, 1, 1, 0, 0, 0, 0],
+         [0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
          [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]
+         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], 'uint8'
     )
 
     assert_array_equal(img, img_)
@@ -185,15 +188,15 @@ def test_polygon_parallelogram():
 
     img_ = np.array(
         [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
+         [0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
          [0, 1, 1, 1, 0, 0, 0, 0, 0, 0],
-         [0, 1, 1, 1, 1, 1, 0, 0, 0, 0],
-         [0, 1, 1, 1, 1, 1, 0, 0, 0, 0],
-         [0, 1, 1, 1, 1, 1, 0, 0, 0, 0],
-         [0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
+         [0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+         [0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+         [0, 1, 1, 1, 1, 1, 1, 0, 0, 0],
+         [0, 0, 0, 0, 1, 1, 1, 0, 0, 0],
+         [0, 0, 0, 0, 0, 0, 1, 0, 0, 0],
          [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
-         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]
+         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], 'uint8'
     )
 
     assert_array_equal(img, img_)
@@ -212,10 +215,22 @@ def test_polygon_exceed():
     assert_array_equal(img, img_)
 
 
-def test_circle():
+def test_polygon_0d_input():
+    # Bug reported in #4938: 0d input causes segfault.
+    rr, cc = polygon(0, 1)
+
+    assert rr.size == cc.size == 1
+
+
+def test_circle_deprecated():
+    with expected_warnings(['`draw.circle` is deprecated']):
+        _ = circle(7, 7, 6)
+
+
+def test_disk():
     img = np.zeros((15, 15), 'uint8')
 
-    rr, cc = circle(7, 7, 6)
+    rr, cc = disk((7, 7), 6)
     img[rr, cc] = 1
 
     img_ = np.array(
