@@ -100,17 +100,17 @@ class ORB(FeatureDetector, DescriptorExtractor):
            [3, 3],
            [4, 4]])
     >>> detector_extractor1.keypoints[matches[:, 0]]
-    array([[ 42.,  40.],
-           [ 47.,  58.],
-           [ 44.,  40.],
-           [ 59.,  42.],
-           [ 45.,  44.]])
+    array([[42., 40.],
+           [47., 58.],
+           [44., 40.],
+           [59., 42.],
+           [45., 44.]])
     >>> detector_extractor2.keypoints[matches[:, 1]]
-    array([[ 55.,  53.],
-           [ 60.,  71.],
-           [ 57.,  53.],
-           [ 72.,  55.],
-           [ 58.,  57.]])
+    array([[55., 53.],
+           [60., 71.],
+           [57., 53.],
+           [72., 55.],
+           [58., 57.]])
 
     """
 
@@ -136,15 +136,16 @@ class ORB(FeatureDetector, DescriptorExtractor):
                                      self.downscale, multichannel=False))
 
     def _detect_octave(self, octave_image):
+        dtype = octave_image.dtype
         # Extract keypoints for current octave
         fast_response = corner_fast(octave_image, self.fast_n,
                                     self.fast_threshold)
         keypoints = corner_peaks(fast_response, min_distance=1)
 
         if len(keypoints) == 0:
-            return (np.zeros((0, 2), dtype=np.double),
-                    np.zeros((0, ), dtype=np.double),
-                    np.zeros((0, ), dtype=np.double))
+            return (np.zeros((0, 2), dtype=dtype),
+                    np.zeros((0, ), dtype=dtype),
+                    np.zeros((0, ), dtype=dtype))
 
         mask = _mask_border_keypoints(octave_image.shape, keypoints,
                                       distance=16)
@@ -181,13 +182,14 @@ class ORB(FeatureDetector, DescriptorExtractor):
 
             octave_image = np.ascontiguousarray(pyramid[octave])
 
-            keypoints, orientations, responses = \
-                self._detect_octave(octave_image)
+            keypoints, orientations, responses = self._detect_octave(
+                octave_image)
 
             keypoints_list.append(keypoints * self.downscale ** octave)
             orientations_list.append(orientations)
             scales_list.append(np.full(
-                keypoints.shape[0], self.downscale ** octave, dtype='float64'))
+                keypoints.shape[0], self.downscale ** octave,
+                dtype=octave_image.dtype))
             responses_list.append(responses)
 
         keypoints = np.vstack(keypoints_list)
@@ -213,7 +215,7 @@ class ORB(FeatureDetector, DescriptorExtractor):
                                       distance=20)
         keypoints = np.array(keypoints[mask], dtype=np.intp, order='C',
                              copy=False)
-        orientations = np.array(orientations[mask], dtype=np.double, order='C',
+        orientations = np.array(orientations[mask], order='C',
                                 copy=False)
 
         descriptors = _orb_loop(octave_image, keypoints, orientations)
@@ -261,7 +263,6 @@ class ORB(FeatureDetector, DescriptorExtractor):
 
                 octave_keypoints = keypoints[octave_mask]
                 octave_keypoints /= self.downscale ** octave
-
                 octave_orientations = orientations[octave_mask]
 
                 descriptors, mask = self._extract_octave(octave_image,
@@ -271,7 +272,7 @@ class ORB(FeatureDetector, DescriptorExtractor):
                 descriptors_list.append(descriptors)
                 mask_list.append(mask)
 
-        self.descriptors = np.vstack(descriptors_list).view(np.bool)
+        self.descriptors = np.vstack(descriptors_list).view(bool)
         self.mask_ = np.hstack(mask_list)
 
     def detect_and_extract(self, image):
@@ -300,13 +301,13 @@ class ORB(FeatureDetector, DescriptorExtractor):
 
             octave_image = np.ascontiguousarray(pyramid[octave])
 
-            keypoints, orientations, responses = \
-                self._detect_octave(octave_image)
+            keypoints, orientations, responses = self._detect_octave(
+                octave_image)
 
             if len(keypoints) == 0:
                 keypoints_list.append(keypoints)
                 responses_list.append(responses)
-                descriptors_list.append(np.zeros((0, 256), dtype=np.bool))
+                descriptors_list.append(np.zeros((0, 256), dtype=bool))
                 continue
 
             descriptors, mask = self._extract_octave(octave_image, keypoints,
@@ -329,7 +330,7 @@ class ORB(FeatureDetector, DescriptorExtractor):
         responses = np.hstack(responses_list)
         scales = np.hstack(scales_list)
         orientations = np.hstack(orientations_list)
-        descriptors = np.vstack(descriptors_list).view(np.bool)
+        descriptors = np.vstack(descriptors_list).view(bool)
 
         if keypoints.shape[0] < self.n_keypoints:
             self.keypoints = keypoints
