@@ -1,11 +1,12 @@
 import numpy as np
-from skimage._shared.testing import assert_array_equal
-from skimage._shared.testing import assert_almost_equal, assert_warns
+
+from skimage._shared.testing import (assert_almost_equal, assert_array_equal,
+                                     assert_equal)
 from skimage import data
 from skimage import img_as_float
 from skimage import draw
 from skimage.color import rgb2gray
-from skimage.morphology import octagon
+from skimage.morphology import cube, octagon
 from skimage._shared.testing import test_parallel
 from skimage._shared._warnings import expected_warnings
 from skimage._shared import testing
@@ -16,6 +17,7 @@ from skimage.feature import (corner_moravec, corner_harris, corner_shi_tomasi,
                              corner_kitchen_rosenfeld, corner_foerstner,
                              corner_fast, corner_orientations,
                              structure_tensor, structure_tensor_eigvals,
+                             structure_tensor_eigenvalues,
                              hessian_matrix, hessian_matrix_eigvals,
                              hessian_matrix_det, shape_index)
 
@@ -32,22 +34,64 @@ def im3d():
 def test_structure_tensor():
     square = np.zeros((5, 5))
     square[2, 2] = 1
-    Axx, Axy, Ayy = structure_tensor(square, sigma=0.1)
-    assert_array_equal(Axx, np.array([[ 0,  0,  0,  0,  0],
-                                      [ 0,  1,  0,  1,  0],
-                                      [ 0,  4,  0,  4,  0],
-                                      [ 0,  1,  0,  1,  0],
-                                      [ 0,  0,  0,  0,  0]]))
-    assert_array_equal(Axy, np.array([[ 0,  0,  0,  0,  0],
-                                      [ 0,  1,  0, -1,  0],
-                                      [ 0,  0,  0, -0,  0],
-                                      [ 0, -1, -0,  1,  0],
-                                      [ 0,  0,  0,  0,  0]]))
-    assert_array_equal(Ayy, np.array([[ 0,  0,  0,  0,  0],
-                                      [ 0,  1,  4,  1,  0],
-                                      [ 0,  0,  0,  0,  0],
-                                      [ 0,  1,  4,  1,  0],
-                                      [ 0,  0,  0,  0,  0]]))
+    Arr, Arc, Acc = structure_tensor(square, sigma=0.1, order='rc')
+    assert_array_equal(Acc, np.array([[0, 0, 0, 0, 0],
+                                      [0, 1, 0, 1, 0],
+                                      [0, 4, 0, 4, 0],
+                                      [0, 1, 0, 1, 0],
+                                      [0, 0, 0, 0, 0]]))
+    assert_array_equal(Arc, np.array([[0, 0, 0, 0, 0],
+                                      [0, 1, 0, -1, 0],
+                                      [0, 0, 0, -0, 0],
+                                      [0, -1, -0, 1, 0],
+                                      [0, 0, 0, 0, 0]]))
+    assert_array_equal(Arr, np.array([[0, 0, 0, 0, 0],
+                                      [0, 1, 4, 1, 0],
+                                      [0, 0, 0, 0, 0],
+                                      [0, 1, 4, 1, 0],
+                                      [0, 0, 0, 0, 0]]))
+
+
+def test_structure_tensor_3d():
+    cube = np.zeros((5, 5, 5))
+    cube[2, 2, 2] = 1
+    A_elems = structure_tensor(cube, sigma=0.1)
+    assert_equal(len(A_elems), 6)
+    assert_array_equal(A_elems[0][:, 1, :], np.array([[0, 0, 0, 0, 0],
+                                                      [0, 1, 4, 1, 0],
+                                                      [0, 0, 0, 0, 0],
+                                                      [0, 1, 4, 1, 0],
+                                                      [0, 0, 0, 0, 0]]))
+    assert_array_equal(A_elems[0][1], np.array([[0, 0, 0, 0, 0],
+                                                [0, 1, 4, 1, 0],
+                                                [0, 4, 16, 4, 0],
+                                                [0, 1, 4, 1, 0],
+                                                [0, 0, 0, 0, 0]]))
+    assert_array_equal(A_elems[3][2], np.array([[0, 0, 0, 0, 0],
+                                                [0, 4, 16, 4, 0],
+                                                [0, 0, 0, 0, 0],
+                                                [0, 4, 16, 4, 0],
+                                                [0, 0, 0, 0, 0]]))
+
+
+def test_structure_tensor_3d_rc_only():
+    cube = np.zeros((5, 5, 5))
+    with testing.raises(ValueError):
+        structure_tensor(cube, sigma=0.1, order='xy')
+    A_elems_rc = structure_tensor(cube, sigma=0.1, order='rc')
+    A_elems_none = structure_tensor(cube, sigma=0.1)
+    assert_array_equal(A_elems_rc, A_elems_none)
+
+
+def test_structure_tensor_orders():
+    square = np.zeros((5, 5))
+    square[2, 2] = 1
+    with expected_warnings(['the default order of the structure']):
+        A_elems_default = structure_tensor(square, sigma=0.1)
+    A_elems_xy = structure_tensor(square, sigma=0.1, order='xy')
+    A_elems_rc = structure_tensor(square, sigma=0.1, order='rc')
+    assert_array_equal(A_elems_xy, A_elems_default)
+    assert_array_equal(A_elems_xy, A_elems_rc[::-1])
 
 
 def test_hessian_matrix():
@@ -86,11 +130,11 @@ def test_hessian_matrix_3d():
                                                   [0,  0,  0,  0,  0]]))
 
 
-def test_structure_tensor_eigvals():
+def test_structure_tensor_eigenvalues():
     square = np.zeros((5, 5))
     square[2, 2] = 1
-    Axx, Axy, Ayy = structure_tensor(square, sigma=0.1)
-    l1, l2 = structure_tensor_eigvals(Axx, Axy, Ayy)
+    A_elems = structure_tensor(square, sigma=0.1, order='rc')
+    l1, l2 = structure_tensor_eigenvalues(A_elems)
     assert_array_equal(l1, np.array([[0, 0, 0, 0, 0],
                                      [0, 2, 4, 2, 0],
                                      [0, 4, 0, 4, 0],
@@ -103,6 +147,26 @@ def test_structure_tensor_eigvals():
                                      [0, 0, 0, 0, 0]]))
 
 
+def test_structure_tensor_eigenvalues_3d():
+    image = np.pad(cube(9), 5, mode='constant') * 1000
+    boundary = (np.pad(cube(9), 5, mode='constant')
+                - np.pad(cube(7), 6, mode='constant')).astype(bool)
+    A_elems = structure_tensor(image, sigma=0.1)
+    e0, e1, e2 = structure_tensor_eigenvalues(A_elems)
+    # e0 should detect facets
+    assert np.all(e0[boundary] != 0)
+
+
+def test_structure_tensor_eigvals():
+    square = np.zeros((5, 5))
+    square[2, 2] = 1
+    A_elems = structure_tensor(square, sigma=0.1, order='rc')
+    with expected_warnings(['structure_tensor_eigvals is deprecated']):
+        eigvals = structure_tensor_eigvals(*A_elems)
+    eigenvalues = structure_tensor_eigenvalues(A_elems)
+    assert_array_equal(eigvals, eigenvalues)
+
+
 def test_hessian_matrix_eigvals():
     square = np.zeros((5, 5))
     square[2, 2] = 4
@@ -187,10 +251,9 @@ def test_square_image():
     im[:25, :25] = 1.
 
     # Moravec
-    results = peak_local_max(corner_moravec(im),
-                             min_distance=10, threshold_rel=0)
+    results = corner_moravec(im) > 0
     # interest points along edge
-    assert len(results) == 57
+    assert np.count_nonzero(results) == 92
 
     # Harris
     results = peak_local_max(corner_harris(im, method='k'),
@@ -263,28 +326,22 @@ def test_rotated_img():
     im_rotated = im.T
 
     # Moravec
-    results = peak_local_max(corner_moravec(im),
-                             min_distance=10, threshold_rel=0)
-    results_rotated = peak_local_max(corner_moravec(im_rotated),
-                                     min_distance=10, threshold_rel=0)
-    assert (np.sort(results[:, 0]) == np.sort(results_rotated[:, 1])).all()
-    assert (np.sort(results[:, 1]) == np.sort(results_rotated[:, 0])).all()
+    results = np.nonzero(corner_moravec(im))
+    results_rotated = np.nonzero(corner_moravec(im_rotated))
+    assert (np.sort(results[0]) == np.sort(results_rotated[1])).all()
+    assert (np.sort(results[1]) == np.sort(results_rotated[0])).all()
 
     # Harris
-    results = peak_local_max(corner_harris(im),
-                             min_distance=10, threshold_rel=0)
-    results_rotated = peak_local_max(corner_harris(im_rotated),
-                                     min_distance=10, threshold_rel=0)
-    assert (np.sort(results[:, 0]) == np.sort(results_rotated[:, 1])).all()
-    assert (np.sort(results[:, 1]) == np.sort(results_rotated[:, 0])).all()
+    results = np.nonzero(corner_harris(im))
+    results_rotated = np.nonzero(corner_harris(im_rotated))
+    assert (np.sort(results[0]) == np.sort(results_rotated[1])).all()
+    assert (np.sort(results[1]) == np.sort(results_rotated[0])).all()
 
     # Shi-Tomasi
-    results = peak_local_max(corner_shi_tomasi(im),
-                             min_distance=10, threshold_rel=0)
-    results_rotated = peak_local_max(corner_shi_tomasi(im_rotated),
-                                     min_distance=10, threshold_rel=0)
-    assert (np.sort(results[:, 0]) == np.sort(results_rotated[:, 1])).all()
-    assert (np.sort(results[:, 1]) == np.sort(results_rotated[:, 0])).all()
+    results = np.nonzero(corner_shi_tomasi(im))
+    results_rotated = np.nonzero(corner_shi_tomasi(im_rotated))
+    assert (np.sort(results[0]) == np.sort(results_rotated[1])).all()
+    assert (np.sort(results[1]) == np.sort(results_rotated[0])).all()
 
 
 def test_subpix_edge():
@@ -320,17 +377,18 @@ def test_subpix_no_class():
 
 def test_subpix_border():
     img = np.zeros((50, 50))
-    img[1:25,1:25] = 255
-    img[25:-1,25:-1] = 255
+    img[1:25, 1:25] = 255
+    img[25:-1, 25:-1] = 255
     corner = corner_peaks(corner_harris(img), threshold_rel=0)
     subpix = corner_subpix(img, corner, window_size=11)
-    ref = np.array([[ 0.52040816,  0.52040816],
-                    [ 0.52040816, 24.47959184],
-                    [24.47959184,  0.52040816],
-                    [24.5       , 24.5       ],
+    ref = np.array([[24.5, 24.5],
+                    [0.52040816, 0.52040816],
+                    [0.52040816, 24.47959184],
+                    [24.47959184, 0.52040816],
                     [24.52040816, 48.47959184],
                     [48.47959184, 24.52040816],
                     [48.47959184, 48.47959184]])
+
     assert_almost_equal(subpix, ref)
 
 
@@ -351,17 +409,22 @@ def test_num_peaks():
 def test_corner_peaks():
     response = np.zeros((10, 10))
     response[2:5, 2:5] = 1
+    response[8:10, 0:2] = 1
 
     corners = corner_peaks(response, exclude_border=False, min_distance=10,
                            threshold_rel=0)
-    assert len(corners) == 1
+    assert corners.shape == (1, 2)
+
+    corners = corner_peaks(response, exclude_border=False, min_distance=5,
+                           threshold_rel=0)
+    assert corners.shape == (2, 2)
 
     corners = corner_peaks(response, exclude_border=False, min_distance=1)
-    assert len(corners) == 4
+    assert corners.shape == (5, 2)
 
     corners = corner_peaks(response, exclude_border=False, min_distance=1,
                            indices=False)
-    assert np.sum(corners) == 4
+    assert np.sum(corners) == 5
 
 
 def test_blank_image_nans():
@@ -388,43 +451,43 @@ def test_corner_fast_image_unsupported_error():
 @test_parallel()
 def test_corner_fast_astronaut():
     img = rgb2gray(data.astronaut())
-    expected = np.array([[101, 198],
-                        [140, 205],
-                        [141, 242],
-                        [177, 156],
-                        [188, 113],
-                        [197, 148],
-                        [213, 117],
-                        [223, 375],
-                        [232, 266],
-                        [245, 137],
-                        [249, 171],
-                        [300, 244],
-                        [305,  57],
-                        [325, 245],
-                        [339, 242],
-                        [346, 279],
-                        [353, 172],
-                        [358, 307],
-                        [362, 252],
-                        [362, 328],
-                        [363, 192],
-                        [364, 147],
-                        [369, 159],
-                        [374, 171],
-                        [379, 183],
-                        [387, 195],
-                        [390, 149],
-                        [401, 197],
-                        [403, 162],
-                        [413, 181],
-                        [444, 310],
-                        [464, 251],
-                        [476, 250],
-                        [489, 155],
-                        [492, 139],
-                        [494, 169],
-                        [496, 266]])
+    expected = np.array([[444, 310],
+                         [374, 171],
+                         [249, 171],
+                         [492, 139],
+                         [403, 162],
+                         [496, 266],
+                         [362, 328],
+                         [476, 250],
+                         [353, 172],
+                         [346, 279],
+                         [494, 169],
+                         [177, 156],
+                         [413, 181],
+                         [213, 117],
+                         [390, 149],
+                         [140, 205],
+                         [232, 266],
+                         [489, 155],
+                         [387, 195],
+                         [101, 198],
+                         [363, 192],
+                         [364, 147],
+                         [300, 244],
+                         [325, 245],
+                         [141, 242],
+                         [401, 197],
+                         [197, 148],
+                         [339, 242],
+                         [188, 113],
+                         [362, 252],
+                         [379, 183],
+                         [358, 307],
+                         [245, 137],
+                         [369, 159],
+                         [464, 251],
+                         [305,  57],
+                         [223, 375]])
     actual = corner_peaks(corner_fast(img, 12, 0.3),
                           min_distance=10, threshold_rel=0)
     assert_array_equal(actual, expected)
@@ -451,18 +514,25 @@ def test_corner_orientations_astronaut():
     img = rgb2gray(data.astronaut())
     corners = corner_peaks(corner_fast(img, 11, 0.35),
                            min_distance=10, threshold_abs=0, threshold_rel=0.1)
-    expected = np.array([-1.75220190e+00,  2.01197383e+00, -2.01162417e+00,
-                         -1.88247204e-01,  1.19134149e+00, -6.61151410e-01,
-                         -2.99143370e+00,  2.17103132e+00, -7.52950306e-04,
-                          1.25854853e+00,  2.43573659e+00, -1.69230287e+00,
-                         -9.88548213e-01,  1.47154532e+00, -1.65449964e+00,
-                          1.09650167e+00,  1.07812134e+00, -1.68885773e+00,
-                         -1.64397304e+00,  3.09780364e+00, -3.49561988e-01,
-                         -1.46554357e+00, -2.81524886e+00,  8.12701702e-01,
-                          2.47305654e+00, -1.63869275e+00,  5.46905279e-02,
-                         -4.40598471e-01,  3.14918803e-01, -1.76069982e+00,
-                          3.05330950e+00,  2.39291733e+00, -1.22091334e-01,
-                         -3.09279990e-01,  1.45931342e+00])
+    expected = np.array([-4.40598471e-01, -1.46554357e+00,
+                         2.39291733e+00, -1.63869275e+00,
+                         1.45931342e+00, -1.64397304e+00,
+                         -1.76069982e+00, 1.09650167e+00,
+                         -1.65449964e+00, 1.19134149e+00,
+                         5.46905279e-02, 2.17103132e+00,
+                         8.12701702e-01, -1.22091334e-01,
+                         -2.01162417e+00, 1.25854853e+00,
+                         3.05330950e+00, 2.01197383e+00,
+                         1.07812134e+00, 3.09780364e+00,
+                         -3.49561988e-01, 2.43573659e+00,
+                         3.14918803e-01, -9.88548213e-01,
+                         -1.88247204e-01, 2.47305654e+00,
+                         -2.99143370e+00, 1.47154532e+00,
+                         -6.61151410e-01, -1.68885773e+00,
+                         -3.09279990e-01, -2.81524886e+00,
+                         -1.75220190e+00, -1.69230287e+00,
+                         -7.52950306e-04])
+
     actual = corner_orientations(img, corners, octagon(3, 2))
     assert_almost_equal(actual, expected)
 
@@ -474,6 +544,6 @@ def test_corner_orientations_square():
                            min_distance=1, threshold_rel=0)
     actual_orientations = corner_orientations(square, corners, octagon(3, 2))
     actual_orientations_degrees = np.rad2deg(actual_orientations)
-    expected_orientations_degree = np.array([  45.,  135.,  -45., -135.])
+    expected_orientations_degree = np.array([45, 135, -45, -135])
     assert_array_equal(actual_orientations_degrees,
                        expected_orientations_degree)
