@@ -1,5 +1,8 @@
+import itertools
 import numpy as np
+import pytest
 import unittest
+from skimage._shared._warnings import expected_warnings
 from skimage._shared.testing import assert_array_almost_equal
 from skimage._shared.testing import assert_equal
 from scipy import ndimage as ndi
@@ -11,11 +14,13 @@ np.random.seed(21)
 class TestPeakLocalMax():
     def test_trivial_case(self):
         trivial = np.zeros((25, 25))
-        peak_indices = peak.peak_local_max(trivial, min_distance=1, indices=True)
+        peak_indices = peak.peak_local_max(trivial, min_distance=1)
         assert type(peak_indices) is np.ndarray
         assert peak_indices.size == 0
-        peaks = peak.peak_local_max(trivial, min_distance=1, indices=False)
-        assert (peaks.astype(np.bool) == trivial).all()
+        with expected_warnings(["indices argument is deprecated"]):
+            peaks = peak.peak_local_max(trivial, min_distance=1,
+                                        indices=False)
+        assert (peaks.astype(bool) == trivial).all()
 
     def test_noisy_peaks(self):
         peak_locations = [(7, 7), (7, 13), (13, 7), (13, 13)]
@@ -63,10 +68,10 @@ class TestPeakLocalMax():
         image[1, 1] = 20
         image[3, 3] = 10
         peaks = peak.peak_local_max(image, min_distance=1)
-        assert peaks.tolist() == [[3, 3], [1, 1]]
+        assert peaks.tolist() == [[1, 1], [3, 3]]
 
         image = np.zeros((3, 10))
-        image[1, (1, 3, 5, 7)] = (1, 3, 2, 4)
+        image[1, (1, 3, 5, 7)] = (1, 2, 3, 4)
         peaks = peak.peak_local_max(image, min_distance=1)
         assert peaks.tolist() == [[1, 7], [1, 5], [1, 3], [1, 1]]
 
@@ -114,19 +119,16 @@ class TestPeakLocalMax():
         labels = 1 + (i >= 10) + (j >= 15) * 2
         result = peak.peak_local_max(image, labels=labels,
                                      min_distance=1, threshold_rel=0,
-                                     indices=True,
                                      num_peaks=np.inf,
                                      num_peaks_per_label=2)
         assert len(result) == 8
         result = peak.peak_local_max(image, labels=labels,
                                      min_distance=1, threshold_rel=0,
-                                     indices=True,
                                      num_peaks=np.inf,
                                      num_peaks_per_label=1)
         assert len(result) == 4
         result = peak.peak_local_max(image, labels=labels,
                                      min_distance=1, threshold_rel=0,
-                                     indices=True,
                                      num_peaks=2,
                                      num_peaks_per_label=2)
         assert len(result) == 2
@@ -153,9 +155,10 @@ class TestPeakLocalMax():
                 expected[imin:imax, jmin:jmax] = ndi.maximum_filter(
                                 image[imin:imax, jmin:jmax], footprint=footprint)
         expected = (expected == image)
-        result = peak.peak_local_max(image, labels=labels, min_distance=1,
-                                     threshold_rel=0, footprint=footprint,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels, min_distance=1,
+                                         threshold_rel=0, footprint=footprint,
+                                         indices=False, exclude_border=False)
         assert (result == expected).all()
 
     def test_indices_with_labels(self):
@@ -169,74 +172,90 @@ class TestPeakLocalMax():
             for jmin, jmax in ((0, 30), (30, 60)):
                 expected[imin:imax, jmin:jmax] = ndi.maximum_filter(
                     image[imin:imax, jmin:jmax], footprint=footprint)
-        expected = np.transpose(np.nonzero(expected == image))
+        expected = np.stack(np.nonzero(expected == image), axis=-1)
         expected = expected[np.argsort(image[tuple(expected.T)])[::-1]]
         result = peak.peak_local_max(image, labels=labels, min_distance=1,
                                      threshold_rel=0, footprint=footprint,
-                                     indices=True, exclude_border=False)
+                                     exclude_border=False)
         result = result[np.argsort(image[tuple(result.T)])[::-1]]
         assert (result == expected).all()
 
     def test_ndarray_indices_false(self):
         nd_image = np.zeros((5, 5, 5))
         nd_image[2, 2, 2] = 1
-        peaks = peak.peak_local_max(nd_image, min_distance=1, indices=False)
-        assert (peaks == nd_image.astype(np.bool)).all()
+        with expected_warnings(["indices argument is deprecated"]):
+            peaks = peak.peak_local_max(nd_image, min_distance=1,
+                                        indices=False)
+        assert (peaks == nd_image.astype(bool)).all()
 
     def test_ndarray_exclude_border(self):
         nd_image = np.zeros((5, 5, 5))
         nd_image[[1, 0, 0], [0, 1, 0], [0, 0, 1]] = 1
         nd_image[3, 0, 0] = 1
         nd_image[2, 2, 2] = 1
-        expected = np.zeros_like(nd_image, dtype=np.bool)
+        expected = np.zeros_like(nd_image, dtype=bool)
         expected[2, 2, 2] = True
-        expectedNoBorder = nd_image > 0
-        result = peak.peak_local_max(nd_image, min_distance=2,
-            exclude_border=2, indices=False)
-        assert_equal(result, expected)
-        # Check that bools work as expected
-        assert_equal(
-            peak.peak_local_max(nd_image, min_distance=2,
-                exclude_border=2, indices=False),
-            peak.peak_local_max(nd_image, min_distance=2,
-                exclude_border=True, indices=False)
+        expectedNoBorder = np.zeros_like(nd_image, dtype=bool)
+        expectedNoBorder[2, 2, 2] = True
+        expectedNoBorder[0, 0, 1] = True
+        expectedNoBorder[3, 0, 0] = True
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(nd_image, min_distance=2,
+                                         exclude_border=2, indices=False)
+            assert_equal(result, expected)
+            # Check that bools work as expected
+            assert_equal(
+                peak.peak_local_max(nd_image, min_distance=2,
+                                    exclude_border=2, indices=False),
+                peak.peak_local_max(nd_image, min_distance=2,
+                                    exclude_border=True, indices=False)
             )
-        assert_equal(
-            peak.peak_local_max(nd_image, min_distance=2,
-                exclude_border=0, indices=False),
-            peak.peak_local_max(nd_image, min_distance=2,
-                exclude_border=False, indices=False)
+            assert_equal(
+                peak.peak_local_max(nd_image, min_distance=2,
+                                    exclude_border=0, indices=False),
+                peak.peak_local_max(nd_image, min_distance=2,
+                                    exclude_border=False, indices=False)
             )
-        # Check both versions with  no border
-        assert_equal(
-            peak.peak_local_max(nd_image, min_distance=2,
-                exclude_border=0, indices=False),
-            expectedNoBorder,
+            # Check both versions with  no border
+            assert_equal(
+                peak.peak_local_max(nd_image, min_distance=2,
+                                    exclude_border=0, indices=False),
+                expectedNoBorder,
             )
-        assert_equal(
-            peak.peak_local_max(nd_image,
-                exclude_border=False, indices=False),
-            expectedNoBorder,
+            assert_equal(
+                peak.peak_local_max(nd_image,
+                                    exclude_border=False, indices=False),
+                nd_image.astype(bool)
             )
 
     def test_empty(self):
         image = np.zeros((10, 20))
         labels = np.zeros((10, 20), int)
-        result = peak.peak_local_max(image, labels=labels,
-                                     footprint=np.ones((3, 3), bool),
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         footprint=np.ones((3, 3), bool),
+                                         min_distance=1, threshold_rel=0,
+                                         indices=False, exclude_border=False)
         assert np.all(~ result)
 
+    def test_empty_non2d_indices(self):
+        image = np.zeros((10, 10, 10))
+        result = peak.peak_local_max(image,
+                                     footprint=np.ones((3, 3, 3), bool),
+                                     min_distance=1, threshold_rel=0,
+                                     exclude_border=False)
+        assert result.shape == (0, image.ndim)
+
     def test_one_point(self):
         image = np.zeros((10, 20))
         labels = np.zeros((10, 20), int)
         image[5, 5] = 1
         labels[5, 5] = 1
-        result = peak.peak_local_max(image, labels=labels,
-                                     footprint=np.ones((3, 3), bool),
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         footprint=np.ones((3, 3), bool),
+                                         min_distance=1, threshold_rel=0,
+                                         indices=False, exclude_border=False)
         assert np.all(result == (labels == 1))
 
     def test_adjacent_and_same(self):
@@ -244,10 +263,11 @@ class TestPeakLocalMax():
         labels = np.zeros((10, 20), int)
         image[5, 5:6] = 1
         labels[5, 5:6] = 1
-        result = peak.peak_local_max(image, labels=labels,
-                                     footprint=np.ones((3, 3), bool),
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         footprint=np.ones((3, 3), bool),
+                                         min_distance=1, threshold_rel=0,
+                                         indices=False, exclude_border=False)
         assert np.all(result == (labels == 1))
 
     def test_adjacent_and_different(self):
@@ -257,14 +277,16 @@ class TestPeakLocalMax():
         image[5, 6] = .5
         labels[5, 5:6] = 1
         expected = (image == 1)
-        result = peak.peak_local_max(image, labels=labels,
-                                     footprint=np.ones((3, 3), bool),
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         footprint=np.ones((3, 3), bool),
+                                         min_distance=1, threshold_rel=0,
+                                         indices=False, exclude_border=False)
         assert np.all(result == expected)
-        result = peak.peak_local_max(image, labels=labels,
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         min_distance=1, threshold_rel=0,
+                                         indices=False, exclude_border=False)
         assert np.all(result == expected)
 
     def test_not_adjacent_and_different(self):
@@ -274,10 +296,11 @@ class TestPeakLocalMax():
         image[5, 8] = .5
         labels[image > 0] = 1
         expected = (labels == 1)
-        result = peak.peak_local_max(image, labels=labels,
-                                     footprint=np.ones((3, 3), bool),
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         footprint=np.ones((3, 3), bool),
+                                         min_distance=1, threshold_rel=0,
+                                         indices=False, exclude_border=False)
         assert np.all(result == expected)
 
     def test_two_objects(self):
@@ -288,10 +311,11 @@ class TestPeakLocalMax():
         labels[5, 5] = 1
         labels[5, 15] = 2
         expected = (labels > 0)
-        result = peak.peak_local_max(image, labels=labels,
-                                     footprint=np.ones((3, 3), bool),
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         footprint=np.ones((3, 3), bool),
+                                         min_distance=1, threshold_rel=0,
+                                         indices=False, exclude_border=False)
         assert np.all(result == expected)
 
     def test_adjacent_different_objects(self):
@@ -302,10 +326,11 @@ class TestPeakLocalMax():
         labels[5, 5] = 1
         labels[5, 6] = 2
         expected = (labels > 0)
-        result = peak.peak_local_max(image, labels=labels,
-                                     footprint=np.ones((3, 3), bool),
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         footprint=np.ones((3, 3), bool),
+                                         min_distance=1, threshold_rel=0,
+                                         indices=False, exclude_border=False)
         assert np.all(result == expected)
 
     def test_four_quadrants(self):
@@ -320,9 +345,13 @@ class TestPeakLocalMax():
                 expected[imin:imax, jmin:jmax] = ndi.maximum_filter(
                     image[imin:imax, jmin:jmax], footprint=footprint)
         expected = (expected == image)
-        result = peak.peak_local_max(image, labels=labels, footprint=footprint,
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=labels,
+                                         footprint=footprint,
+                                         min_distance=1,
+                                         threshold_rel=0,
+                                         indices=False,
+                                         exclude_border=False)
         assert np.all(result == expected)
 
     def test_disk(self):
@@ -331,14 +360,18 @@ class TestPeakLocalMax():
         '''
         image = np.random.uniform(size=(10, 20))
         footprint = np.array([[1]])
-        result = peak.peak_local_max(image, labels=np.ones((10, 20)),
-                                     footprint=footprint,
-                                     min_distance=1, threshold_rel=0,
-                                     threshold_abs=-1, indices=False,
-                                     exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, labels=np.ones((10, 20), int),
+                                         footprint=footprint,
+                                         min_distance=1, threshold_rel=0,
+                                         threshold_abs=-1, indices=False,
+                                         exclude_border=False)
         assert np.all(result)
-        result = peak.peak_local_max(image, footprint=footprint, threshold_abs=-1,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            result = peak.peak_local_max(image, footprint=footprint,
+                                         threshold_abs=-1,
+                                         indices=False,
+                                         exclude_border=False)
         assert np.all(result)
 
     def test_3D(self):
@@ -371,7 +404,6 @@ class TestPeakLocalMax():
                                           threshold_rel=0).tolist()) == \
             [[5, 5, 5, 5], [15, 15, 15, 15]]
 
-
     def test_threshold_rel_default(self):
         image = np.ones((5, 5))
 
@@ -382,7 +414,78 @@ class TestPeakLocalMax():
         assert_equal(peak.peak_local_max(image), [[2, 2]])
 
         image[2, 2] = 0
-        assert len(peak.peak_local_max(image, min_distance=0)) == image.size - 1
+        with expected_warnings(["When min_distance < 1"]):
+            assert len(peak.peak_local_max(image,
+                                           min_distance=0)) == image.size - 1
+
+
+@pytest.mark.parametrize(
+    ["indices"],
+    [[indices] for indices in itertools.product(range(5), range(5))],
+)
+def test_exclude_border(indices):
+    image = np.zeros((5, 5))
+    image[indices] = 1
+
+    # exclude_border = False, means it will always be found.
+    assert len(peak.peak_local_max(image, exclude_border=False)) == 1
+
+    # exclude_border = 0, means it will always be found.
+    assert len(peak.peak_local_max(image, exclude_border=0)) == 1
+
+    # exclude_border = True, min_distance=1 means it will be found unless it's
+    # on the edge.
+    if indices[0] in (0, 4) or indices[1] in (0, 4):
+        expected_peaks = 0
+    else:
+        expected_peaks = 1
+    assert len(peak.peak_local_max(
+        image, min_distance=1, exclude_border=True)) == expected_peaks
+
+    # exclude_border = (1, 0) means it will be found unless it's on the edge of
+    # the first dimension.
+    if indices[0] in (0, 4):
+        expected_peaks = 0
+    else:
+        expected_peaks = 1
+    assert len(peak.peak_local_max(
+        image, exclude_border=(1, 0))) == expected_peaks
+
+    # exclude_border = (0, 1) means it will be found unless it's on the edge of
+    # the second dimension.
+    if indices[1] in (0, 4):
+        expected_peaks = 0
+    else:
+        expected_peaks = 1
+    assert len(peak.peak_local_max(
+        image, exclude_border=(0, 1))) == expected_peaks
+
+
+def test_exclude_border_errors():
+    image = np.zeros((5, 5))
+
+    # exclude_border doesn't have the right cardinality.
+    with pytest.raises(ValueError):
+        assert peak.peak_local_max(image, exclude_border=(1,))
+
+    # exclude_border doesn't have the right type
+    with pytest.raises(TypeError):
+        assert peak.peak_local_max(image, exclude_border=1.0)
+
+    # exclude_border is a tuple of the right cardinality but contains
+    # non-integer values.
+    with pytest.raises(ValueError):
+        assert peak.peak_local_max(image, exclude_border=(1, 'a'))
+
+    # exclude_border is a tuple of the right cardinality but contains a
+    # negative value.
+    with pytest.raises(ValueError):
+        assert peak.peak_local_max(image, exclude_border=(1, -1))
+
+    # exclude_border is a negative value.
+    with pytest.raises(ValueError):
+        assert peak.peak_local_max(image, exclude_border=-1)
+
 
 class TestProminentPeaks(unittest.TestCase):
     def test_isolated_peaks(self):
@@ -429,7 +532,7 @@ class TestProminentPeaks(unittest.TestCase):
         image[y1, x1] = i1
         image[y2, x2] = i2
         out = peak._prominent_peaks(image, min_xdistance=3,
-                                   min_ydistance=3,)
+                                    min_ydistance=3,)
         assert_equal(out[0], np.array((i1,)))
         assert_equal(out[1], np.array((x1,)))
         assert_equal(out[2], np.array((y1,)))
@@ -438,12 +541,13 @@ class TestProminentPeaks(unittest.TestCase):
         image = np.zeros((10, 20))
         labels = np.zeros((10, 20), int)
         image[5, 5] = 1
-        labels[5, 5] = 1
+        labels[5, 5] = 3
         labelsin = labels.copy()
-        result = peak.peak_local_max(image, labels=labels,
-                                     footprint=np.ones((3, 3), bool),
-                                     min_distance=1, threshold_rel=0,
-                                     indices=False, exclude_border=False)
+        with expected_warnings(["indices argument is deprecated"]):
+            peak.peak_local_max(image, labels=labels,
+                                footprint=np.ones((3, 3), bool),
+                                min_distance=1, threshold_rel=0,
+                                indices=False, exclude_border=False)
         assert np.all(labels == labelsin)
 
     def test_many_objects(self):
@@ -454,6 +558,6 @@ class TestProminentPeaks(unittest.TestCase):
         mask[(x - x_c) ** 2 + (y - y_c) ** 2 < 8 ** 2] = True
         labels, num_objs = ndi.label(mask)
         dist = ndi.distance_transform_edt(mask)
-        local_max = peak.peak_local_max(dist, min_distance=20, indices=True,
+        local_max = peak.peak_local_max(dist, min_distance=20,
                                         exclude_border=False, labels=labels)
         assert len(local_max) == 625
