@@ -6,7 +6,16 @@ production code that will depend on updated skimage versions.
 """
 
 from . import graph
-from .manual_segmentation import manual_polygon_segmentation, manual_lasso_segmentation
+from .manual_segmentation import manual_polygon_segmentation
+from .manual_segmentation import manual_lasso_segmentation
+from .trainable_segmentation import fit_segmenter, predict_segmenter, TrainableSegmenter
 
 
-__all__ = ['graph', 'manual_lasso_segmentation', 'manual_polygon_segmentation']
+__all__ = [
+    "graph",
+    "manual_lasso_segmentation",
+    "manual_polygon_segmentation",
+    "fit_segmenter",
+    "predict_segmenter",
+    "TrainableSegmenter",
+]
