@@ -51,7 +51,7 @@ def route_through_array(array, start, end, fully_connected=True,
     ([(0, 0), (0, 1), (1, 1)], 9.5)
     >>> # Now allow diagonal steps: the path goes directly from start to end
     >>> route_through_array(image, [0, 0], [1, 1])
-    ([(0, 0), (1, 1)], 9.1923881554251192)
+    ([(0, 0), (1, 1)], 9.19238815542512)
     >>> # Cost is the sum of array values along the path (16 = 1 + 3 + 12)
     >>> route_through_array(image, [0, 0], [1, 1], fully_connected=False,
     ... geometric=False)
@@ -67,7 +67,7 @@ def route_through_array(array, start, end, fully_connected=True,
            [30, 31, 32, 33, 34, 35]])
     >>> # Find the path with lowest cost
     >>> indices, weight = route_through_array(image, (0, 0), (5, 5))
-    >>> indices = np.array(indices).T
+    >>> indices = np.stack(indices, axis=-1)
     >>> path = np.zeros_like(image)
     >>> path[indices[0], indices[1]] = 1
     >>> path
