@@ -1,8 +1,9 @@
 import os
 
 import numpy as np
+import imageio
 from skimage import data_dir
-from skimage.io.collection import ImageCollection, alphanumeric_key
+from skimage.io.collection import ImageCollection, MultiImage, alphanumeric_key
 from skimage.io import reset_plugins
 
 from skimage._shared import testing
@@ -18,11 +19,10 @@ def test_string_split():
 def test_string_sort():
     filenames = ['f9.10.png', 'f9.9.png', 'f10.10.png', 'f10.9.png',
                  'e9.png', 'e10.png', 'em.png']
-    sorted_filenames = ['e9.png', 'e10.png', 'em.png', 'f9.9.png',
-                        'f9.10.png', 'f10.9.png', 'f10.10.png']
+    expected_filenames = ['e9.png', 'e10.png', 'em.png', 'f9.9.png',
+                          'f9.10.png', 'f10.9.png', 'f10.10.png']
     sorted_filenames = sorted(filenames, key=alphanumeric_key)
-    assert_equal(sorted_filenames, sorted_filenames)
-
+    assert_equal(expected_filenames, sorted_filenames)
 
 def test_imagecollection_input():
     """Test function for ImageCollection. The new behavior (implemented
@@ -33,9 +33,12 @@ def test_imagecollection_input():
     -----
         If correct, `images` will receive three images.
     """
+    # Ensure that these images are part of the legacy datasets
+    # this means they will always be available in the user's install
+    # regarless of the availability of pooch
     pattern = [os.path.join(data_dir, pic)
-               for pic in ['palette_gray.png',
-                           'chessboard_GRAY_U16.tif',
+               for pic in ['coffee.png',
+                           'chessboard_GRAY.png',
                            'rocket.jpg']]
     images = ImageCollection(pattern)
     assert len(images) == 3
@@ -43,10 +46,10 @@ def test_imagecollection_input():
 
 class TestImageCollection(TestCase):
     pattern = [os.path.join(data_dir, pic)
-               for pic in ['camera.png', 'color.png']]
+               for pic in ['brick.png', 'color.png']]
 
     pattern_matched = [os.path.join(data_dir, pic)
-                       for pic in ['camera.png', 'moon.png']]
+                       for pic in ['brick.png', 'moon.png']]
 
     def setUp(self):
         reset_plugins()
@@ -54,6 +57,8 @@ class TestImageCollection(TestCase):
         self.images = ImageCollection(self.pattern)
         # Image collection with images having shapes that match.
         self.images_matched = ImageCollection(self.pattern_matched)
+        # Same images as a collection of frames
+        self.frames_matched = MultiImage(self.pattern_matched)
 
     def test_len(self):
         assert len(self.images) == 2
@@ -90,6 +95,20 @@ class TestImageCollection(TestCase):
         with testing.raises(AttributeError):
             set_files('newfiles')
 
+    def test_custom_load_func_w_kwarg(self):
+        load_pattern = os.path.join(data_dir, 'no_time_for_that_tiny.gif')
+
+        def load_fn(f, step):
+            vid = imageio.get_reader(f)
+            seq = [v for v in vid.iter_data()]
+            return seq[::step]
+
+        ic = ImageCollection(load_pattern, load_func=load_fn, step=3)
+        # Each file should map to one image (array).
+        assert len(ic) == 1
+        # GIF file has 24 frames, so 24 / 3 equals 8.
+        assert len(ic[0]) == 8
+
     def test_custom_load_func(self):
 
         def load_fn(x):
@@ -106,3 +125,7 @@ class TestImageCollection(TestCase):
     def test_concatenate_mismatched_image_shapes(self):
         with testing.raises(ValueError):
             self.images.concatenate()
+
+    def test_multiimage_imagecollection(self):
+        assert_equal(self.images_matched[0], self.frames_matched[0])
+        assert_equal(self.images_matched[1], self.frames_matched[1])
