@@ -1,13 +1,11 @@
-try:
-    from urllib.request import urlopen  # Python 3
-except ImportError:
-    from urllib2 import urlopen  # Python 2
+import urllib.parse
+import urllib.request
+from urllib.error import URLError, HTTPError
 
 import os
 import re
 import tempfile
 from contextlib import contextmanager
-import six
 
 
 URL_REGEX = re.compile(r'http://|https://|ftp://|file://|file:\\')
@@ -15,7 +13,7 @@ URL_REGEX = re.compile(r'http://|https://|ftp://|file://|file:\\')
 
 def is_url(filename):
     """Return True if string is an http or ftp path."""
-    return (isinstance(filename, six.string_types) and
+    return (isinstance(filename, str) and
             URL_REGEX.match(filename) is not None)
 
 
@@ -23,14 +21,23 @@ def is_url(filename):
 def file_or_url_context(resource_name):
     """Yield name of file from the given resource (i.e. file or url)."""
     if is_url(resource_name):
-        _, ext = os.path.splitext(resource_name)
+        url_components = urllib.parse.urlparse(resource_name)
+        _, ext = os.path.splitext(url_components.path)
         try:
             with tempfile.NamedTemporaryFile(delete=False, suffix=ext) as f:
-                u = urlopen(resource_name)
+                u = urllib.request.urlopen(resource_name)
                 f.write(u.read())
             # f must be closed before yielding
             yield f.name
-        finally:
+        except (URLError, HTTPError):
+            # could not open URL
+            os.remove(f.name)
+            raise
+        except (FileNotFoundError, FileExistsError,
+                PermissionError, BaseException):
+            # could not create temporary file
+            raise
+        else:
             os.remove(f.name)
     else:
         yield resource_name
