@@ -83,10 +83,10 @@ def moments_coords_central(coords, center=None, order=3):
     ...                    for row in range(13, 17)
     ...                    for col in range(14, 18)])
     >>> moments_coords_central(coords)
-    array([[ 16.,   0.,  20.,   0.],
-           [  0.,   0.,   0.,   0.],
-           [ 20.,   0.,  25.,   0.],
-           [  0.,   0.,   0.,   0.]])
+    array([[16.,  0., 20.,  0.],
+           [ 0.,  0.,  0.,  0.],
+           [20.,  0., 25.,  0.],
+           [ 0.,  0.,  0.,  0.]])
 
     As seen above, for symmetric objects, odd-order moments (columns 1 and 3,
     rows 1 and 3) are zero when centered on the centroid, or center of mass,
@@ -96,10 +96,10 @@ def moments_coords_central(coords, center=None, order=3):
     >>> coords2 = np.concatenate((coords, [[17, 17]]), axis=0)
     >>> np.round(moments_coords_central(coords2),
     ...          decimals=2)  # doctest: +NORMALIZE_WHITESPACE
-    array([[ 17.  ,   0.  ,  22.12,  -2.49],
-           [  0.  ,   3.53,   1.73,   7.4 ],
-           [ 25.88,   6.02,  36.63,   8.83],
-           [  4.15,  19.17,  14.8 ,  39.6 ]])
+    array([[17.  ,  0.  , 22.12, -2.49],
+           [ 0.  ,  3.53,  1.73,  7.4 ],
+           [25.88,  6.02, 36.63,  8.83],
+           [ 4.15, 19.17, 14.8 , 39.6 ]])
 
     Image moments and central image moments are equivalent (by definition)
     when the center is (0, 0):
@@ -112,7 +112,7 @@ def moments_coords_central(coords, center=None, order=3):
         # This format corresponds to coordinate tuples as returned by
         # e.g. np.nonzero: (row_coords, column_coords).
         # We represent them as an npoints x ndim array.
-        coords = np.transpose(coords)
+        coords = np.stack(coords, axis=-1)
     check_nD(coords, 2)
     ndim = coords.shape[1]
     if center is None:
@@ -233,10 +233,10 @@ def moments_central(image, center=None, order=3, **kwargs):
     >>> M = moments(image)
     >>> centroid = (M[1, 0] / M[0, 0], M[0, 1] / M[0, 0])
     >>> moments_central(image, centroid)
-    array([[ 16.,   0.,  20.,   0.],
-           [  0.,   0.,   0.,   0.],
-           [ 20.,   0.,  25.,   0.],
-           [  0.,   0.,   0.,   0.]])
+    array([[16.,  0., 20.,  0.],
+           [ 0.,  0.,  0.,  0.],
+           [20.,  0., 25.,  0.],
+           [ 0.,  0.,  0.,  0.]])
     """
     if center is None:
         center = centroid(image)
@@ -288,11 +288,10 @@ def moments_normalized(mu, order=3):
     >>> centroid = (m[0, 1] / m[0, 0], m[1, 0] / m[0, 0])
     >>> mu = moments_central(image, centroid)
     >>> moments_normalized(mu)
-    array([[        nan,         nan,  0.078125  ,  0.        ],
-           [        nan,  0.        ,  0.        ,  0.        ],
-           [ 0.078125  ,  0.        ,  0.00610352,  0.        ],
-           [ 0.        ,  0.        ,  0.        ,  0.        ]])
-
+    array([[       nan,        nan, 0.078125  , 0.        ],
+           [       nan, 0.        , 0.        , 0.        ],
+           [0.078125  , 0.        , 0.00610352, 0.        ],
+           [0.        , 0.        , 0.        , 0.        ]])
     """
     if np.any(np.array(mu.shape) <= order):
         raise ValueError("Shape of image moments must be >= `order`")
@@ -315,7 +314,7 @@ def moments_hu(nu):
     Parameters
     ----------
     nu : (M, M) array
-        Normalized central image moments, where M must be > 4.
+        Normalized central image moments, where M must be >= 4.
 
     Returns
     -------
@@ -335,7 +334,16 @@ def moments_hu(nu):
            Berlin, 1993.
     .. [5] https://en.wikipedia.org/wiki/Image_moment
 
-
+    Examples
+    --------
+    >>> image = np.zeros((20, 20), dtype=np.double)
+    >>> image[13:17, 13:17] = 0.5
+    >>> image[10:12, 10:12] = 1
+    >>> mu = moments_central(image)
+    >>> nu = moments_normalized(mu)
+    >>> moments_hu(nu)
+    array([7.45370370e-01, 3.51165981e-01, 1.04049179e-01, 4.06442107e-02,
+           2.64312299e-03, 2.40854582e-02, 4.33680869e-19])
     """
     return _moments_cy.moments_hu(nu.astype(np.double))
 
@@ -352,6 +360,14 @@ def centroid(image):
     -------
     center : tuple of float, length ``image.ndim``
         The centroid of the (nonzero) pixels in ``image``.
+
+    Examples
+    --------
+    >>> image = np.zeros((20, 20), dtype=np.double)
+    >>> image[13:17, 13:17] = 0.5
+    >>> image[10:12, 10:12] = 1
+    >>> centroid(image)
+    array([13.16666667, 13.16666667])
     """
     M = moments_central(image, center=(0,) * image.ndim, order=1)
     center = (M[tuple(np.eye(image.ndim, dtype=int))]  # array of weighted sums
@@ -445,4 +461,9 @@ def inertia_tensor_eigvals(image, mu=None, T=None):
     if T is None:
         T = inertia_tensor(image, mu)
     eigvals = np.linalg.eigvalsh(T)
+    # Floating point precision problems could make a positive
+    # semidefinite matrix have an eigenvalue that is very slightly
+    # negative. This can cause problems down the line, so set values
+    # very near zero to zero.
+    eigvals = np.clip(eigvals, 0, None, out=eigvals)
     return sorted(eigvals, reverse=True)
