@@ -2,6 +2,7 @@ import math
 import numpy as np
 from numpy.linalg import inv, pinv
 from scipy import optimize
+from warnings import warn
 from .._shared.utils import check_random_state
 
 
@@ -47,18 +48,18 @@ class LineModelND(BaseModel):
     >>> x = np.linspace(1, 2, 25)
     >>> y = 1.5 * x + 3
     >>> lm = LineModelND()
-    >>> lm.estimate(np.array([x, y]).T)
+    >>> lm.estimate(np.stack([x, y], axis=-1))
     True
     >>> tuple(np.round(lm.params, 5))
-    (array([ 1.5 ,  5.25]), array([ 0.5547 ,  0.83205]))
-    >>> res = lm.residuals(np.array([x, y]).T)
+    (array([1.5 , 5.25]), array([0.5547 , 0.83205]))
+    >>> res = lm.residuals(np.stack([x, y], axis=-1))
     >>> np.abs(np.round(res, 9))
-    array([ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
-            0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.])
+    array([0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
+           0., 0., 0., 0., 0., 0., 0., 0.])
     >>> np.round(lm.predict_y(x[:5]), 3)
-    array([ 4.5  ,  4.562,  4.625,  4.688,  4.75 ])
+    array([4.5  , 4.562, 4.625, 4.688, 4.75 ])
     >>> np.round(lm.predict_x(y[:5]), 3)
-    array([ 1.   ,  1.042,  1.083,  1.125,  1.167])
+    array([1.   , 1.042, 1.083, 1.125, 1.167])
 
     """
 
@@ -248,9 +249,8 @@ class CircleModel(BaseModel):
     (2.0, 3.0, 4.0)
     >>> res = model.residuals(xy)
     >>> np.abs(np.round(res, 9))
-    array([ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
-            0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.])
-
+    array([0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
+           0., 0., 0., 0., 0., 0., 0., 0.])
     """
 
     def estimate(self, data):
@@ -278,12 +278,12 @@ class CircleModel(BaseModel):
         sum_x = np.sum(x)
         sum_y = np.sum(y)
         sum_xy = np.sum(x * y)
-        m1 = np.array([[np.sum(x ** 2), sum_xy, sum_x],
+        m1 = np.stack([[np.sum(x ** 2), sum_xy, sum_x],
                        [sum_xy, np.sum(y ** 2), sum_y],
                        [sum_x, sum_y, float(len(x))]])
-        m2 = np.array([[np.sum(x * x2y2),
+        m2 = np.stack([[np.sum(x * x2y2),
                         np.sum(y * x2y2),
-                        np.sum(x2y2)]]).T
+                        np.sum(x2y2)]], axis=-1)
         a, b, c = pinv(m1) @ m2
         a, b, c = a[0], b[0], c[0]
         xc = a / 2
@@ -382,10 +382,10 @@ class EllipseModel(BaseModel):
     >>> ellipse.estimate(xy)
     True
     >>> np.round(ellipse.params, 2)
-    array([ 10.  ,  15.  ,   4.  ,   8.  ,   0.52])
+    array([10.  , 15.  ,  4.  ,  8.  ,  0.52])
     >>> np.round(abs(ellipse.residuals(xy)), 5)
-    array([ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,
-            0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.])
+    array([0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
+           0., 0., 0., 0., 0., 0., 0., 0.])
     """
 
     def estimate(self, data):
@@ -692,8 +692,8 @@ def ransac(data, model_class, min_samples, residual_threshold,
             N >= log(1 - probability) / log(1 - e**m)
 
         where the probability (confidence) is typically set to a high value
-        such as 0.99, and e is the current fraction of inliers w.r.t. the
-        total number of samples.
+        such as 0.99, e is the current fraction of inliers w.r.t. the
+        total number of samples, and m is the min_samples value.
     random_state : int, RandomState instance or None, optional
         If int, random_state is the seed used by the random number generator;
         If RandomState instance, random_state is the random number generator;
@@ -747,7 +747,7 @@ def ransac(data, model_class, min_samples, residual_threshold,
 
     >>> ransac_model, inliers = ransac(data, EllipseModel, 20, 3, max_trials=50)
     >>> abs(np.round(ransac_model.params))
-    array([ 20.,  30.,   5.,  10.,   0.])
+    array([20., 30.,  5., 10.,  0.])
     >>> inliers # doctest: +SKIP
     array([False, False, False, False,  True,  True,  True,  True,  True,
             True,  True,  True,  True,  True,  True,  True,  True,  True,
@@ -779,7 +779,7 @@ def ransac(data, model_class, min_samples, residual_threshold,
             True,  True,  True,  True,  True,  True,  True,  True,  True,
             True,  True,  True,  True,  True,  True,  True,  True,  True,
             True,  True,  True,  True,  True,  True,  True,  True,  True,
-            True,  True,  True,  True,  True], dtype=bool)
+            True,  True,  True,  True,  True])
 
     """
 
@@ -869,9 +869,13 @@ def ransac(data, model_class, min_samples, residual_threshold,
                 break
 
     # estimate final model using all inliers
-    if best_inliers is not None:
+    if best_inliers is not None and any(best_inliers):
         # select inliers for each data array
         data_inliers = [d[best_inliers] for d in data]
         best_model.estimate(*data_inliers)
+    else:
+        best_model = None
+        best_inliers = None
+        warn("No inliers found. Model not fitted")
 
     return best_model, best_inliers
