@@ -6,6 +6,8 @@ from numpy.testing import assert_equal
 from pytest import raises, warns
 
 from skimage.morphology import extrema
+from skimage._shared.testing import expected_warnings
+
 
 eps = 1e-12
 
@@ -171,6 +173,133 @@ class TestExtrema(unittest.TestCase):
         error = diff(expected_result, out)
         assert error < eps
 
+    def test_h_maxima_float_image(self):
+        """specific tests for h-maxima float image type"""
+        w = 10
+        x, y = np.mgrid[0:w, 0:w]
+        data = 20 - 0.2 * ((x - w / 2) ** 2 + (y - w / 2) ** 2)
+        data[2:4, 2:4] = 40
+        data[2:4, 7:9] = 60
+        data[7:9, 2:4] = 80
+        data[7:9, 7:9] = 100
+        data = data.astype(np.float32)
+
+        expected_result = np.zeros_like(data)
+        expected_result[(data > 19.9)] = 1.0
+
+        for h in [1.0e-12, 1.0e-6, 1.0e-3, 1.0e-2, 1.0e-1, 0.1]:
+            out = extrema.h_maxima(data, h)
+            error = diff(expected_result, out)
+            assert error < eps
+
+    def test_h_maxima_float_h(self):
+        """specific tests for h-maxima float h parameter"""
+        data = np.array([[0, 0, 0, 0, 0],
+                         [0, 3, 3, 3, 0],
+                         [0, 3, 4, 3, 0],
+                         [0, 3, 3, 3, 0],
+                         [0, 0, 0, 0, 0]], dtype=np.uint8)
+
+        h_vals = np.linspace(1.0, 2.0, 100)
+        failures = 0
+
+        for h in h_vals:
+            if h % 1 != 0:
+                msgs = ['possible precision loss converting image']
+            else:
+                msgs = []
+
+            with expected_warnings(msgs):
+                maxima = extrema.h_maxima(data, h)
+
+            if (maxima[2, 2] == 0):
+                failures += 1
+
+        assert (failures == 0)
+
+    def test_h_maxima_large_h(self):
+        """test that h-maxima works correctly for large h"""
+        data = np.array([[10, 10, 10, 10, 10],
+                         [10, 13, 13, 13, 10],
+                         [10, 13, 14, 13, 10],
+                         [10, 13, 13, 13, 10],
+                         [10, 10, 10, 10, 10]], dtype=np.uint8)
+
+        maxima = extrema.h_maxima(data, 5)
+        assert (np.sum(maxima) == 0)
+
+        data = np.array([[10, 10, 10, 10, 10],
+                         [10, 13, 13, 13, 10],
+                         [10, 13, 14, 13, 10],
+                         [10, 13, 13, 13, 10],
+                         [10, 10, 10, 10, 10]], dtype=np.float32)
+
+        maxima = extrema.h_maxima(data, 5.0)
+        assert (np.sum(maxima) == 0)
+
+    def test_h_minima_float_image(self):
+        """specific tests for h-minima float image type"""
+        w = 10
+        x, y = np.mgrid[0:w, 0:w]
+        data = 180 + 0.2 * ((x - w / 2) ** 2 + (y - w / 2) ** 2)
+        data[2:4, 2:4] = 160
+        data[2:4, 7:9] = 140
+        data[7:9, 2:4] = 120
+        data[7:9, 7:9] = 100
+        data = data.astype(np.float32)
+
+        expected_result = np.zeros_like(data)
+        expected_result[(data < 180.1)] = 1.0
+
+        for h in [1.0e-12, 1.0e-6, 1.0e-3, 1.0e-2, 1.0e-1, 0.1]:
+            out = extrema.h_minima(data, h)
+            error = diff(expected_result, out)
+            assert error < eps
+
+    def test_h_minima_float_h(self):
+        """specific tests for h-minima float h parameter"""
+        data = np.array([[4, 4, 4, 4, 4],
+                         [4, 1, 1, 1, 4],
+                         [4, 1, 0, 1, 4],
+                         [4, 1, 1, 1, 4],
+                         [4, 4, 4, 4, 4]], dtype=np.uint8)
+
+        h_vals = np.linspace(1.0, 2.0, 100)
+        failures = 0
+        for h in h_vals:
+            if h % 1 != 0:
+                msgs = ['possible precision loss converting image']
+            else:
+                msgs = []
+
+            with expected_warnings(msgs):
+                minima = extrema.h_minima(data, h)
+
+            if (minima[2, 2] == 0):
+                failures += 1
+
+        assert (failures == 0)
+
+    def test_h_minima_large_h(self):
+        """test that h-minima works correctly for large h"""
+        data = np.array([[14, 14, 14, 14, 14],
+                         [14, 11, 11, 11, 14],
+                         [14, 11, 10, 11, 14],
+                         [14, 11, 11, 11, 14],
+                         [14, 14, 14, 14, 14]], dtype=np.uint8)
+
+        maxima = extrema.h_minima(data, 5)
+        assert (np.sum(maxima) == 0)
+
+        data = np.array([[14, 14, 14, 14, 14],
+                         [14, 11, 11, 11, 14],
+                         [14, 11, 10, 11, 14],
+                         [14, 11, 11, 11, 14],
+                         [14, 14, 14, 14, 14]], dtype=np.float32)
+
+        maxima = extrema.h_minima(data, 5.0)
+        assert (np.sum(maxima) == 0)
+
 
 class TestLocalMaxima(unittest.TestCase):
     """Some tests for local_minima are included as well."""
@@ -197,7 +326,7 @@ class TestLocalMaxima(unittest.TestCase):
          [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0],
          [0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0],
          [0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0]],
-        dtype=np.bool
+        dtype=bool
     )
     # Connectivity 1 (cross), maxima can touch border
     expected_cross = np.array(
@@ -207,14 +336,14 @@ class TestLocalMaxima(unittest.TestCase):
          [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0],
          [0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0],
          [0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0]],
-        dtype=np.bool
+        dtype=bool
     )
 
     def test_empty(self):
         """Test result with empty image."""
         result = extrema.local_maxima(np.array([[]]), indices=False)
         assert result.size == 0
-        assert result.dtype == np.bool
+        assert result.dtype == bool
         assert result.shape == (1, 0)
 
         result = extrema.local_maxima(np.array([]), indices=True)
@@ -235,7 +364,7 @@ class TestLocalMaxima(unittest.TestCase):
         """Test results with default configuration for all supported dtypes."""
         for dtype in self.supported_dtypes:
             result = extrema.local_maxima(self.image.astype(dtype))
-            assert result.dtype == np.bool
+            assert result.dtype == bool
             assert_equal(result, self.expected_default)
 
     def test_dtypes_old(self):
@@ -267,53 +396,53 @@ class TestLocalMaxima(unittest.TestCase):
              [0, 0, 1, 1, 0, 0, 0, 1, 1, 0],
              [0, 0, 1, 1, 0, 0, 0, 1, 1, 0],
              [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]],
-            dtype=np.bool
+            dtype=bool
         )
         for dtype in self.supported_dtypes:
             image = data.astype(dtype)
             result = extrema.local_maxima(image)
-            assert result.dtype == np.bool
+            assert result.dtype == bool
             assert_equal(result, expected)
 
     def test_connectivity(self):
         """Test results if selem is a scalar."""
         # Connectivity 1: generates cross shaped structuring element
         result_conn1 = extrema.local_maxima(self.image, connectivity=1)
-        assert result_conn1.dtype == np.bool
+        assert result_conn1.dtype == bool
         assert_equal(result_conn1, self.expected_cross)
 
         # Connectivity 2: generates square shaped structuring element
         result_conn2 = extrema.local_maxima(self.image, connectivity=2)
-        assert result_conn2.dtype == np.bool
+        assert result_conn2.dtype == bool
         assert_equal(result_conn2, self.expected_default)
 
         # Connectivity 3: generates square shaped structuring element
         result_conn3 = extrema.local_maxima(self.image, connectivity=3)
-        assert result_conn3.dtype == np.bool
+        assert result_conn3.dtype == bool
         assert_equal(result_conn3, self.expected_default)
 
     def test_selem(self):
         """Test results if selem is given."""
         selem_cross = np.array(
-            [[0, 1, 0], [1, 1, 1], [0, 1, 0]], dtype=np.bool)
+            [[0, 1, 0], [1, 1, 1], [0, 1, 0]], dtype=bool)
         result_selem_cross = extrema.local_maxima(
             self.image, selem=selem_cross)
-        assert result_selem_cross.dtype == np.bool
+        assert result_selem_cross.dtype == bool
         assert_equal(result_selem_cross, self.expected_cross)
 
         for selem in [
             ((True,) * 3,) * 3,
             np.ones((3, 3), dtype=np.float64),
             np.ones((3, 3), dtype=np.uint8),
-            np.ones((3, 3), dtype=np.bool),
+            np.ones((3, 3), dtype=bool),
         ]:
             # Test different dtypes for selem which expects a boolean array but
             # will accept and convert other types if possible
             result_selem_square = extrema.local_maxima(self.image, selem=selem)
-            assert result_selem_square.dtype == np.bool
+            assert result_selem_square.dtype == bool
             assert_equal(result_selem_square, self.expected_default)
 
-        selem_x = np.array([[1, 0, 1], [0, 1, 0], [1, 0, 1]], dtype=np.bool)
+        selem_x = np.array([[1, 0, 1], [0, 1, 0], [1, 0, 1]], dtype=bool)
         expected_selem_x = np.array(
             [[1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0],
              [1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
@@ -321,10 +450,10 @@ class TestLocalMaxima(unittest.TestCase):
              [0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0],
              [0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0],
              [0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0]],
-            dtype=np.bool
+            dtype=bool
         )
         result_selem_x = extrema.local_maxima(self.image, selem=selem_x)
-        assert result_selem_x.dtype == np.bool
+        assert result_selem_x.dtype == bool
         assert_equal(result_selem_x, expected_selem_x)
 
     def test_indices(self):
@@ -347,7 +476,7 @@ class TestLocalMaxima(unittest.TestCase):
         # of interest
         result_with_boder = extrema.local_maxima(
             self.image, connectivity=1, allow_borders=True)
-        assert result_with_boder.dtype == np.bool
+        assert result_with_boder.dtype == bool
         assert_equal(result_with_boder, self.expected_cross)
 
         expected_without_border = np.array(
@@ -357,11 +486,11 @@ class TestLocalMaxima(unittest.TestCase):
              [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0],
              [0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]],
-            dtype=np.bool
+            dtype=bool
         )
         result_without_border = extrema.local_maxima(
             self.image, connectivity=1, allow_borders=False)
-        assert result_with_boder.dtype == np.bool
+        assert result_with_boder.dtype == bool
         assert_equal(result_without_border, expected_without_border)
 
     def test_nd(self):
@@ -369,14 +498,14 @@ class TestLocalMaxima(unittest.TestCase):
         # One-dimension
         x_1d = np.array([1, 1, 0, 1, 2, 3, 0, 2, 1, 2, 0])
         expected_1d = np.array([1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0],
-                               dtype=np.bool)
+                               dtype=bool)
         result_1d = extrema.local_maxima(x_1d)
-        assert result_1d.dtype == np.bool
+        assert result_1d.dtype == bool
         assert_equal(result_1d, expected_1d)
 
         # 3-dimensions (adapted from old unit test)
         x_3d = np.zeros((8, 8, 8), dtype=np.uint8)
-        expected_3d = np.zeros((8, 8, 8), dtype=np.bool)
+        expected_3d = np.zeros((8, 8, 8), dtype=bool)
         # first maximum: only one pixel
         x_3d[1, 1:3, 1:3] = 100
         x_3d[2, 2, 2] = 200
@@ -397,7 +526,7 @@ class TestLocalMaxima(unittest.TestCase):
         x_3d[7, 7, 7] = 255
         expected_3d[7, 7, 7] = 1
         result_3d = extrema.local_maxima(x_3d)
-        assert result_3d.dtype == np.bool
+        assert result_3d.dtype == bool
         assert_equal(result_3d, expected_3d)
 
     def test_constant(self):
@@ -408,11 +537,11 @@ class TestLocalMaxima(unittest.TestCase):
             const_image = const_image.astype(dtype)
             # test for local maxima
             result = extrema.local_maxima(const_image)
-            assert result.dtype == np.bool
+            assert result.dtype == bool
             assert_equal(result, expected)
             # test for local minima
             result = extrema.local_minima(const_image)
-            assert result.dtype == np.bool
+            assert result.dtype == bool
             assert_equal(result, expected)
 
     def test_extrema_float(self):
@@ -443,17 +572,17 @@ class TestLocalMaxima(unittest.TestCase):
              [0, 0, 1, 1, 0, 0, 0, 1, 1, 0],
              [0, 0, 1, 1, 0, 0, 0, 1, 1, 0],
              [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]],
-            dtype=np.bool
+            dtype=bool
         )
 
         # Test for local maxima with automatic step calculation
         result = extrema.local_maxima(image)
-        assert result.dtype == np.bool
+        assert result.dtype == bool
         assert_equal(result, expected_result)
 
         # Test for local minima with automatic step calculation
         result = extrema.local_minima(inverted_image)
-        assert result.dtype == np.bool
+        assert result.dtype == bool
         assert_equal(result, expected_result)
 
     def test_exceptions(self):
@@ -461,18 +590,18 @@ class TestLocalMaxima(unittest.TestCase):
         # Mismatching number of dimensions
         with raises(ValueError, match="number of dimensions"):
             extrema.local_maxima(
-                self.image, selem=np.ones((3, 3, 3), dtype=np.bool))
+                self.image, selem=np.ones((3, 3, 3), dtype=bool))
         with raises(ValueError, match="number of dimensions"):
             extrema.local_maxima(
-                self.image, selem=np.ones((3,), dtype=np.bool))
+                self.image, selem=np.ones((3,), dtype=bool))
 
         # All dimensions in selem must be of size 3
         with raises(ValueError, match="dimension size"):
             extrema.local_maxima(
-                self.image, selem=np.ones((2, 3), dtype=np.bool))
+                self.image, selem=np.ones((2, 3), dtype=bool))
         with raises(ValueError, match="dimension size"):
             extrema.local_maxima(
-                self.image, selem=np.ones((5, 5), dtype=np.bool))
+                self.image, selem=np.ones((5, 5), dtype=bool))
 
         with raises(TypeError, match="float16 which is not supported"):
             extrema.local_maxima(np.empty(1, dtype=np.float16))
@@ -494,7 +623,7 @@ class TestLocalMaxima(unittest.TestCase):
         with warns(UserWarning, match=warning_msg):
             result = extrema.local_maxima(x, allow_borders=False)
         assert_equal(result, [0, 0])
-        assert result.dtype == np.bool
+        assert result.dtype == bool
 
         x = np.array([[1, 2], [2, 2]])
         extrema.local_maxima(x, allow_borders=True, indices=True)  # no warning
