@@ -23,7 +23,7 @@ def _inpaint_biharmonic_single_channel(mask, out, limits):
     mask_i = np.ravel_multi_index(np.where(mask), mask.shape)
 
     # Find masked points and prepare them to be easily enumerate over
-    mask_pts = np.array(np.where(mask)).T
+    mask_pts = np.stack(np.where(mask), axis=-1)
 
     # Iterate over masked points
     for mask_pt_n, mask_pt_idx in enumerate(mask_pts):
@@ -124,7 +124,7 @@ def inpaint_biharmonic(image, mask, multichannel=False):
         raise TypeError('Masked arrays are not supported')
 
     image = skimage.img_as_float(image)
-    mask = mask.astype(np.bool)
+    mask = mask.astype(bool)
 
     # Split inpainting mask into independent regions
     kernel = ndi.morphology.generate_binary_structure(mask.ndim, 1)
