@@ -11,13 +11,13 @@ def configuration(parent_package='', top_path=None):
     from numpy.distutils.misc_util import Configuration, get_numpy_include_dirs
 
     config = Configuration('restoration', parent_package, top_path)
-    config.add_data_dir('tests')
 
     cython(['_unwrap_1d.pyx',
             '_unwrap_2d.pyx',
             '_unwrap_3d.pyx',
             '_denoise_cy.pyx',
             '_nl_means_denoising.pyx'], working_path=base_path)
+    cython(['_rolling_ball_cy.pyx'], working_path=base_path)
 
     config.add_extension('_unwrap_1d', sources=['_unwrap_1d.c'],
                          include_dirs=[get_numpy_include_dirs()])
@@ -28,14 +28,17 @@ def configuration(parent_package='', top_path=None):
     config.add_extension('_unwrap_3d', sources=unwrap_sources_3d,
                          include_dirs=[get_numpy_include_dirs()])
     config.add_extension('_denoise_cy', sources=['_denoise_cy.c'],
-        include_dirs=[get_numpy_include_dirs(), '../_shared'])
+                         include_dirs=[get_numpy_include_dirs()])
     config.add_extension('_nl_means_denoising',
                          sources=['_nl_means_denoising.c'],
                          include_dirs=[get_numpy_include_dirs(),
                                        '../_shared'])
+    config.add_extension('_rolling_ball_cy', sources=['_rolling_ball_cy.c'],
+                         include_dirs=[get_numpy_include_dirs()])
 
     return config
 
+
 if __name__ == '__main__':
     from numpy.distutils.core import setup
     setup(maintainer='scikit-image Developers',
