@@ -1,4 +1,3 @@
-from __future__ import print_function, division
 
 import numpy as np
 from skimage.restoration import unwrap_phase
@@ -15,18 +14,19 @@ from skimage._shared._warnings import expected_warnings
 def assert_phase_almost_equal(a, b, *args, **kwargs):
     """An assert_almost_equal insensitive to phase shifts of n*2*pi."""
     shift = 2 * np.pi * np.round((b.mean() - a.mean()) / (2 * np.pi))
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")
+    with expected_warnings([r'invalid value encountered|\A\Z',
+                            r'divide by zero encountered|\A\Z']):
         print('assert_phase_allclose, abs', np.max(np.abs(a - (b - shift))))
         print('assert_phase_allclose, rel',
               np.max(np.abs((a - (b - shift)) / a)))
     if np.ma.isMaskedArray(a):
         assert_(np.ma.isMaskedArray(b))
         assert_array_equal(a.mask, b.mask)
+        assert_(a.fill_value == b.fill_value)
         au = np.asarray(a)
         bu = np.asarray(b)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
+        with expected_warnings([r'invalid value encountered|\A\Z',
+                                r'divide by zero encountered|\A\Z']):
             print('assert_phase_allclose, no mask, abs',
                   np.max(np.abs(au - (bu - shift))))
             print('assert_phase_allclose, no mask, rel',
@@ -38,8 +38,8 @@ def check_unwrap(image, mask=None):
     image_wrapped = np.angle(np.exp(1j * image))
     if mask is not None:
         print('Testing a masked image')
-        image = np.ma.array(image, mask=mask)
-        image_wrapped = np.ma.array(image_wrapped, mask=mask)
+        image = np.ma.array(image, mask=mask, fill_value=0.5)
+        image_wrapped = np.ma.array(image_wrapped, mask=mask, fill_value=0.5)
     image_unwrapped = unwrap_phase(image_wrapped, seed=0)
     assert_phase_almost_equal(image_unwrapped, image)
 
@@ -61,7 +61,7 @@ def test_unwrap_2d(check_with_mask):
     x, y = np.ogrid[:8, :16]
     image = 2 * np.pi * (x * 0.2 + y * 0.1)
     if check_with_mask:
-        mask = np.zeros(image.shape, dtype=np.bool)
+        mask = np.zeros(image.shape, dtype=bool)
         mask[4:6, 4:8] = True
     check_unwrap(image, mask)
 
@@ -72,7 +72,7 @@ def test_unwrap_3d(check_with_mask):
     x, y, z = np.ogrid[:8, :12, :16]
     image = 2 * np.pi * (x * 0.2 + y * 0.1 + z * 0.05)
     if check_with_mask:
-        mask = np.zeros(image.shape, dtype=np.bool)
+        mask = np.zeros(image.shape, dtype=bool)
         mask[4:6, 4:6, 1:3] = True
     check_unwrap(image, mask)
 
@@ -89,9 +89,8 @@ def check_wrap_around(ndim, axis):
     index_first = tuple([0] * ndim)
     index_last = tuple([-1 if n == axis else 0 for n in range(ndim)])
     # unwrap the image without wrap around
-    with warnings.catch_warnings():
-        # We do not want warnings about length 1 dimensions
-        warnings.simplefilter("ignore")
+    # We do not want warnings about length 1 dimensions
+    with expected_warnings([r'Image has a length 1 dimension|\A\Z']):
         image_unwrap_no_wrap_around = unwrap_phase(image_wrapped, seed=0)
     print('endpoints without wrap_around:',
           image_unwrap_no_wrap_around[index_first],
@@ -101,9 +100,8 @@ def check_wrap_around(ndim, axis):
                 image_unwrap_no_wrap_around[index_last]) > np.pi)
     # unwrap the image with wrap around
     wrap_around = [n == axis for n in range(ndim)]
-    with warnings.catch_warnings():
-        # We do not want warnings about length 1 dimensions
-        warnings.simplefilter("ignore")
+    # We do not want warnings about length 1 dimensions
+    with expected_warnings([r'Image has a length 1 dimension.|\A\Z']):
         image_unwrap_wrap_around = unwrap_phase(image_wrapped, wrap_around,
                                                 seed=0)
     print('endpoints with wrap_around:',
@@ -130,11 +128,11 @@ def test_mask():
              np.linspace(0, 8 * np.pi, length),
              np.linspace(0, 6 * np.pi, length)]
     image = np.vstack(ramps)
-    mask_1d = np.ones((length,), dtype=np.bool)
+    mask_1d = np.ones((length,), dtype=bool)
     mask_1d[0] = mask_1d[-1] = False
     for i in range(len(ramps)):
         # mask all ramps but the i'th one
-        mask = np.zeros(image.shape, dtype=np.bool)
+        mask = np.zeros(image.shape, dtype=bool)
         mask |= mask_1d.reshape(1, -1)
         mask[i, :] = False   # unmask i'th ramp
         image_wrapped = np.ma.array(np.angle(np.exp(1j * image)), mask=mask)
@@ -177,7 +175,7 @@ def test_unwrap_3d_middle_wrap_around():
 
 def test_unwrap_2d_compressed_mask():
     # ValueError when image is masked array with a compressed mask (no masked
-    # elments).  GitHub issue #1346
+    # elements).  GitHub issue #1346
     image = np.ma.zeros((10, 10))
     unwrap = unwrap_phase(image)
     assert_(np.all(unwrap == 0))
