@@ -30,7 +30,7 @@ try:
             pass
     umfpack.UmfpackContext.__del__ = new_del
     UmfpackContext = umfpack.UmfpackContext()
-except:
+except ImportError:
     UmfpackContext = None
 
 try:
@@ -40,9 +40,8 @@ except ImportError:
     amg_loaded = False
 
 from ..util import img_as_float
-from ..filters import rank_order
 
-from scipy.sparse.linalg import cg
+from scipy.sparse.linalg import cg, spsolve
 import scipy
 from distutils.version import LooseVersion as Version
 import functools
@@ -50,8 +49,6 @@ import functools
 if Version(scipy.__version__) >= Version('1.1'):
     cg = functools.partial(cg, atol=0)
 
-# -----------Laplacian--------------------
-
 
 def _make_graph_edges_3d(n_x, n_y, n_z):
     """Returns a list of edges for a 3D image.
@@ -77,8 +74,8 @@ def _make_graph_edges_3d(n_x, n_y, n_z):
         Graph edges with each column describing a node-id pair.
     """
     vertices = np.arange(n_x * n_y * n_z).reshape((n_x, n_y, n_z))
-    edges_deep = np.vstack((vertices[:, :, :-1].ravel(),
-                            vertices[:, :, 1:].ravel()))
+    edges_deep = np.vstack((vertices[..., :-1].ravel(),
+                            vertices[..., 1:].ravel()))
     edges_right = np.vstack((vertices[:, :-1].ravel(),
                              vertices[:, 1:].ravel()))
     edges_down = np.vstack((vertices[:-1].ravel(), vertices[1:].ravel()))
@@ -86,155 +83,189 @@ def _make_graph_edges_3d(n_x, n_y, n_z):
     return edges
 
 
-def _compute_weights_3d(data, spacing, beta=130, eps=1.e-6,
-                        multichannel=False):
+def _compute_weights_3d(data, spacing, beta, eps, multichannel):
     # Weight calculation is main difference in multispectral version
     # Original gradient**2 replaced with sum of gradients ** 2
-    gradients = 0
-    for channel in range(0, data.shape[-1]):
-        gradients += _compute_gradients_3d(data[..., channel],
-                                           spacing) ** 2
+    gradients = np.concatenate(
+        [np.diff(data[..., 0], axis=ax).ravel() / spacing[ax]
+         for ax in [2, 1, 0] if data.shape[ax] > 1], axis=0) ** 2
+    for channel in range(1, data.shape[-1]):
+        gradients += np.concatenate(
+            [np.diff(data[..., channel], axis=ax).ravel() / spacing[ax]
+             for ax in [2, 1, 0] if data.shape[ax] > 1], axis=0) ** 2
+
     # All channels considered together in this standard deviation
-    beta /= 10 * data.std()
+    scale_factor = -beta / (10 * data.std())
     if multichannel:
         # New final term in beta to give == results in trivial case where
         # multiple identical spectra are passed.
-        beta /= np.sqrt(data.shape[-1])
-    gradients *= beta
-    weights = np.exp(- gradients)
+        scale_factor /= np.sqrt(data.shape[-1])
+    weights = np.exp(scale_factor * gradients)
     weights += eps
-    return weights
+    return -weights
 
 
-def _compute_gradients_3d(data, spacing):
-    gr_deep = np.abs(data[:, :, :-1] - data[:, :, 1:]).ravel() / spacing[2]
-    gr_right = np.abs(data[:, :-1] - data[:, 1:]).ravel() / spacing[1]
-    gr_down = np.abs(data[:-1] - data[1:]).ravel() / spacing[0]
-    return np.r_[gr_deep, gr_right, gr_down]
-
-
-def _make_laplacian_sparse(edges, weights):
-    """
-    Sparse implementation
-    """
-    pixel_nb = edges.max() + 1
-    diag = np.arange(pixel_nb)
-    i_indices = np.hstack((edges[0], edges[1]))
-    j_indices = np.hstack((edges[1], edges[0]))
-    data = np.hstack((-weights, -weights))
-    lap = sparse.coo_matrix((data, (i_indices, j_indices)),
-                            shape=(pixel_nb, pixel_nb))
-    connect = - np.ravel(lap.sum(axis=1))
-    lap = sparse.coo_matrix(
-        (np.hstack((data, connect)), (np.hstack((i_indices, diag)),
-                                      np.hstack((j_indices, diag)))),
-        shape=(pixel_nb, pixel_nb))
-    return lap.tocsr()
-
-
-def _clean_labels_ar(X, labels, copy=False):
-    X = X.astype(labels.dtype)
-    if copy:
-        labels = np.copy(labels)
-    labels = np.ravel(labels)
-    labels[labels == 0] = X
-    return labels
-
-
-def _buildAB(lap_sparse, labels):
-    """
-    Build the matrix A and rhs B of the linear system to solve.
-    A and B are two block of the laplacian of the image graph.
-    """
-    labels = labels[labels >= 0]
-    indices = np.arange(labels.size)
-    unlabeled_indices = indices[labels == 0]
-    seeds_indices = indices[labels > 0]
-    # The following two lines take most of the time in this function
-    B = lap_sparse[unlabeled_indices][:, seeds_indices]
-    lap_sparse = lap_sparse[unlabeled_indices][:, unlabeled_indices]
-    nlabels = labels.max()
-    rhs = []
-    for lab in range(1, nlabels + 1):
-        mask = (labels[seeds_indices] == lab)
-        fs = sparse.csr_matrix(mask)
-        fs = fs.transpose()
-        rhs.append(B * fs)
-    return lap_sparse, rhs
-
-
-def _mask_edges_weights(edges, weights, mask):
-    """
-    Remove edges of the graph connected to masked nodes, as well as
-    corresponding weights of the edges.
-    """
-    mask0 = np.hstack((mask[:, :, :-1].ravel(), mask[:, :-1].ravel(),
-                       mask[:-1].ravel()))
-    mask1 = np.hstack((mask[:, :, 1:].ravel(), mask[:, 1:].ravel(),
-                       mask[1:].ravel()))
-    ind_mask = np.logical_and(mask0, mask1)
-    edges, weights = edges[:, ind_mask], weights[ind_mask]
-    max_node_index = edges.max()
-    # Reassign edges labels to 0, 1, ... edges_number - 1
-    order = np.searchsorted(np.unique(edges.ravel()),
-                            np.arange(max_node_index + 1))
-    edges = order[edges.astype(np.int64)]
-    return edges, weights
-
-
-def _build_laplacian(data, spacing, mask=None, beta=50,
-                     multichannel=False):
-    l_x, l_y, l_z = tuple(data.shape[i] for i in range(3))
+def _build_laplacian(data, spacing, mask, beta, multichannel):
+    l_x, l_y, l_z = data.shape[:3]
     edges = _make_graph_edges_3d(l_x, l_y, l_z)
     weights = _compute_weights_3d(data, spacing, beta=beta, eps=1.e-10,
                                   multichannel=multichannel)
     if mask is not None:
-        edges, weights = _mask_edges_weights(edges, weights, mask)
-    lap = _make_laplacian_sparse(edges, weights)
-    del edges, weights
-    return lap
+        # Remove edges of the graph connected to masked nodes, as well
+        # as corresponding weights of the edges.
+        mask0 = np.hstack([mask[..., :-1].ravel(), mask[:, :-1].ravel(),
+                           mask[:-1].ravel()])
+        mask1 = np.hstack([mask[..., 1:].ravel(), mask[:, 1:].ravel(),
+                           mask[1:].ravel()])
+        ind_mask = np.logical_and(mask0, mask1)
+        edges, weights = edges[:, ind_mask], weights[ind_mask]
+
+        # Reassign edges labels to 0, 1, ... edges_number - 1
+        _, inv_idx = np.unique(edges, return_inverse=True)
+        edges = inv_idx.reshape(edges.shape)
+
+    # Build the sparse linear system
+    pixel_nb = edges.shape[1]
+    i_indices = edges.ravel()
+    j_indices = edges[::-1].ravel()
+    data = np.hstack((weights, weights))
+    lap = sparse.coo_matrix((data, (i_indices, j_indices)),
+                            shape=(pixel_nb, pixel_nb))
+    lap.setdiag(-np.ravel(lap.sum(axis=0)))
+    return lap.tocsr()
 
 
-def _check_isolated_seeds(labels):
+def _build_linear_system(data, spacing, labels, nlabels, mask,
+                         beta, multichannel):
     """
-    Prune isolated seed pixels to prevent labeling errors, and
-    return coordinates and label values of isolated seeds, so
-    that it is possible to put labels back in random walker output.
+    Build the matrix A and rhs B of the linear system to solve.
+    A and B are two block of the laplacian of the image graph.
     """
-    fill = ndi.binary_propagation(labels == 0, mask=(labels >= 0))
-    isolated = np.logical_and(labels > 0, np.logical_not(fill))
-    inds = np.nonzero(isolated)
-    values = labels[inds]
-    labels[inds] = -1
-    return inds, values
-
-
-def _unchanged_labels(labels, return_full_prob=False):
-    """
-    Return the input array of labels, unless ``return_full_prob`` is True,
-    in which case a mask of pixels is returned for each unique label, with the
-    last dimension corresponding to unique labels.
-    """
-    if return_full_prob:
-        # Find and iterate over valid labels
-        unique_labels = np.unique(labels)
-        unique_labels = unique_labels[unique_labels > 0]
-
-        out_labels = np.empty(labels.shape + (len(unique_labels),),
-                              dtype=np.bool)
-        for n, i in enumerate(unique_labels):
-            out_labels[..., n] = (labels == i)
-
+    if mask is None:
+        labels = labels.ravel()
     else:
-        out_labels = labels
-    return out_labels
+        labels = labels[mask]
+
+    indices = np.arange(labels.size)
+    seeds_mask = labels > 0
+    unlabeled_indices = indices[~seeds_mask]
+    seeds_indices = indices[seeds_mask]
+
+    lap_sparse = _build_laplacian(data, spacing, mask=mask,
+                                  beta=beta, multichannel=multichannel)
+
+    rows = lap_sparse[unlabeled_indices, :]
+    lap_sparse = rows[:, unlabeled_indices]
+    B = -rows[:, seeds_indices]
+
+    seeds = labels[seeds_mask]
+    seeds_mask = sparse.csc_matrix(np.hstack(
+        [np.atleast_2d(seeds == lab).T for lab in range(1, nlabels + 1)]))
+    rhs = B.dot(seeds_mask)
+
+    return lap_sparse, rhs
 
 
-# ----------- Random walker algorithm --------------------------------
+def _solve_linear_system(lap_sparse, B, tol, mode):
+
+    if mode is None:
+        mode = 'cg_j'
+
+    if mode == 'cg_mg' and not amg_loaded:
+        warn('"cg_mg" not available, it requires pyamg to be installed. '
+             'The "cg_j" mode will be used instead.',
+             stacklevel=2)
+        mode = 'cg_j'
+
+    if mode == 'bf':
+        X = spsolve(lap_sparse, B.toarray()).T
+    else:
+        maxiter = None
+        if mode == 'cg':
+            if UmfpackContext is None:
+                warn('"cg" mode may be slow because UMFPACK is not available. '
+                     'Consider building Scipy with UMFPACK or use a '
+                     'preconditioned version of CG ("cg_j" or "cg_mg" modes).',
+                     stacklevel=2)
+            M = None
+        elif mode == 'cg_j':
+            M = sparse.diags(1.0 / lap_sparse.diagonal())
+        else:
+            # mode == 'cg_mg'
+            lap_sparse = lap_sparse.tocsr()
+            ml = ruge_stuben_solver(lap_sparse)
+            M = ml.aspreconditioner(cycle='V')
+            maxiter = 30
+        cg_out = [
+            cg(lap_sparse, B[:, i].toarray(), tol=tol, M=M, maxiter=maxiter)
+            for i in range(B.shape[1])]
+        if np.any([info > 0 for _, info in cg_out]):
+            warn("Conjugate gradient convergence to tolerance not achieved. "
+                 "Consider decreasing beta to improve system conditionning.",
+                 stacklevel=2)
+        X = np.asarray([x for x, _ in cg_out])
+
+    return X
 
 
-def random_walker(data, labels, beta=130, mode='bf', tol=1.e-3, copy=True,
-                  multichannel=False, return_full_prob=False, spacing=None):
+def _preprocess(labels):
+
+    label_values, inv_idx = np.unique(labels, return_inverse=True)
+
+    if not (label_values == 0).any():
+        warn('Random walker only segments unlabeled areas, where '
+             'labels == 0. No zero valued areas in labels were '
+             'found. Returning provided labels.',
+             stacklevel=2)
+
+        return labels, None, None, None, None
+
+    # If some labeled pixels are isolated inside pruned zones, prune them
+    # as well and keep the labels for the final output
+
+    null_mask = labels == 0
+    pos_mask = labels > 0
+    mask = labels >= 0
+
+    fill = ndi.binary_propagation(null_mask, mask=mask)
+    isolated = np.logical_and(pos_mask, np.logical_not(fill))
+
+    pos_mask[isolated] = False
+
+    # If the array has pruned zones, be sure that no isolated pixels
+    # exist between pruned zones (they could not be determined)
+    if label_values[0] < 0 or np.any(isolated):
+        isolated = np.logical_and(
+            np.logical_not(ndi.binary_propagation(pos_mask, mask=mask)),
+            null_mask)
+
+        labels[isolated] = -1
+        if np.all(isolated[null_mask]):
+            warn('All unlabeled pixels are isolated, they could not be '
+                 'determined by the random walker algorithm.',
+                 stacklevel=2)
+            return labels, None, None, None, None
+
+        mask[isolated] = False
+        mask = np.atleast_3d(mask)
+    else:
+        mask = None
+
+    # Reorder label values to have consecutive integers (no gaps)
+    zero_idx = np.searchsorted(label_values, 0)
+    labels = np.atleast_3d(inv_idx.reshape(labels.shape) - zero_idx)
+
+    nlabels = label_values[zero_idx + 1:].shape[0]
+
+    inds_isolated_seeds = np.nonzero(isolated)
+    isolated_values = labels[inds_isolated_seeds]
+
+    return labels, nlabels, mask, inds_isolated_seeds, isolated_values
+
+
+def random_walker(data, labels, beta=130, mode='cg_j', tol=1.e-3, copy=True,
+                  multichannel=False, return_full_prob=False, spacing=None,
+                  *, prob_tol=1e-3):
     """Random walker algorithm for segmentation from markers.
 
     Random walker algorithm is implemented for gray-level or multichannel
@@ -260,10 +291,8 @@ def random_walker(data, labels, beta=130, mode='bf', tol=1.e-3, copy=True,
     beta : float, optional
         Penalization coefficient for the random walker motion
         (the greater `beta`, the more difficult the diffusion).
-    mode : string, available options {'cg_mg', 'cg', 'bf'}
+    mode : string, available options {'cg', 'cg_j', 'cg_mg', 'bf'}
         Mode for solving the linear system in the random walker algorithm.
-        If no preference given, automatically attempt to use the fastest
-        option available ('cg_mg' from pyamg >> 'cg' with UMFPACK > 'bf').
 
         - 'bf' (brute force): an LU factorization of the Laplacian is
           computed. This is fast for small images (<1024x1024), but very slow
@@ -272,16 +301,17 @@ def random_walker(data, labels, beta=130, mode='bf', tol=1.e-3, copy=True,
           using the Conjugate Gradient method from scipy.sparse.linalg. This is
           less memory-consuming than the brute force method for large images,
           but it is quite slow.
+        - 'cg_j' (conjugate gradient with Jacobi preconditionner): the
+          Jacobi preconditionner is applyed during the Conjugate
+          gradient method iterations. This may accelerate the
+          convergence of the 'cg' method.
         - 'cg_mg' (conjugate gradient with multigrid preconditioner): a
           preconditioner is computed using a multigrid solver, then the
-          solution is computed with the Conjugate Gradient method.  This mode
-          requires that the pyamg module (http://pyamg.github.io/) is
-          installed. For images of size > 512x512, this is the recommended
-          (fastest) mode.
-
+          solution is computed with the Conjugate Gradient method. This mode
+          requires that the pyamg module is installed.
     tol : float, optional
-        tolerance to achieve when solving the linear system, in
-        cg' and 'cg_mg' modes.
+        Tolerance to achieve when solving the linear system using
+        the conjugate gradient based modes ('cg', 'cg_j' and 'cg_mg').
     copy : bool, optional
         If copy is False, the `labels` array will be overwritten with
         the result of the segmentation. Use copy=False if you want to
@@ -290,25 +320,31 @@ def random_walker(data, labels, beta=130, mode='bf', tol=1.e-3, copy=True,
         If True, input data is parsed as multichannel data (see 'data' above
         for proper input format in this case).
     return_full_prob : bool, optional
-        If True, the probability that a pixel belongs to each of the labels
-        will be returned, instead of only the most likely label.
+        If True, the probability that a pixel belongs to each of the
+        labels will be returned, instead of only the most likely
+        label.
     spacing : iterable of floats, optional
         Spacing between voxels in each spatial dimension. If `None`, then
         the spacing between pixels/voxels in each dimension is assumed 1.
+    prob_tol : float, optional
+        Tolerance on the resulting probability to be in the interval [0, 1].
+        If the tolerance is not satisfied, a warning is displayed.
 
     Returns
     -------
     output : ndarray
-        * If `return_full_prob` is False, array of ints of same shape as
-          `data`, in which each pixel has been labeled according to the marker
-          that reached the pixel first by anisotropic diffusion.
+        * If `return_full_prob` is False, array of ints of same shape
+          and data type as `labels`, in which each pixel has been
+          labeled according to the marker that reached the pixel first
+          by anisotropic diffusion.
         * If `return_full_prob` is True, array of floats of shape
-          `(nlabels, data.shape)`. `output[label_nb, i, j]` is the probability
-          that label `label_nb` reaches the pixel `(i, j)` first.
+          `(nlabels, labels.shape)`. `output[label_nb, i, j]` is the
+          probability that label `label_nb` reaches the pixel `(i, j)`
+          first.
 
-    See also
+    See Also
     --------
-    skimage.morphology.watershed: watershed segmentation
+    skimage.morphology.watershed : watershed segmentation
         A segmentation algorithm based on mathematical morphology
         and "flooding" of regions from markers.
 
@@ -367,7 +403,7 @@ def random_walker(data, labels, beta=130, mode='bf', tol=1.e-3, copy=True,
     >>> np.random.seed(0)
     >>> a = np.zeros((10, 10)) + 0.2 * np.random.rand(10, 10)
     >>> a[5:8, 5:8] += 1
-    >>> b = np.zeros_like(a)
+    >>> b = np.zeros_like(a, dtype=np.int32)
     >>> b[3, 3] = 1  # Marker for first phase
     >>> b[6, 6] = 2  # Marker for second phase
     >>> random_walker(a, b)
@@ -384,31 +420,22 @@ def random_walker(data, labels, beta=130, mode='bf', tol=1.e-3, copy=True,
 
     """
     # Parse input data
-    if mode is None:
-        if amg_loaded:
-            mode = 'cg_mg'
-        elif UmfpackContext is not None:
-            mode = 'cg'
-        else:
-            mode = 'bf'
-    elif mode not in ('cg_mg', 'cg', 'bf'):
-        raise ValueError("{mode} is not a valid mode. Valid modes are 'cg_mg',"
-                         " 'cg' and 'bf'".format(mode=mode))
+    if mode not in ('cg_mg', 'cg', 'bf', 'cg_j', None):
+        raise ValueError(
+            "{mode} is not a valid mode. Valid modes are 'cg_mg',"
+            " 'cg', 'cg_j', 'bf' and None".format(mode=mode))
 
-    if UmfpackContext is None and mode == 'cg':
-        warn('"cg" mode will be used, but it may be slower than '
-             '"bf" because SciPy was built without UMFPACK. Consider'
-             ' rebuilding SciPy with UMFPACK; this will greatly '
-             'accelerate the conjugate gradient ("cg") solver. '
-             'You may also install pyamg and run the random_walker '
-             'function in "cg_mg" mode (see docstring).')
-
-    if (labels != 0).all():
-        warn('Random walker only segments unlabeled areas, where '
-             'labels == 0. No zero valued areas in labels were '
-             'found. Returning provided labels.')
-
-        return _unchanged_labels(labels, return_full_prob)
+    # Spacing kwarg checks
+    if spacing is None:
+        spacing = np.ones(3)
+    elif len(spacing) == labels.ndim:
+        if len(spacing) == 2:
+            # Need a dummy spacing for singleton 3rd dim
+            spacing = np.r_[spacing, 1.]
+        spacing = np.asarray(spacing)
+    else:
+        raise ValueError('Input argument `spacing` incorrect, should be an '
+                         'iterable with one number per spatial dimension.')
 
     # This algorithm expects 4-D arrays of floats, where the first three
     # dimensions are spatial and the final denotes channels. 2-D images have
@@ -420,154 +447,68 @@ def random_walker(data, labels, beta=130, mode='bf', tol=1.e-3, copy=True,
         if data.ndim not in (2, 3):
             raise ValueError('For non-multichannel input, data must be of '
                              'dimension 2 or 3.')
-        dims = data.shape  # To reshape final labeled result
+        if data.shape != labels.shape:
+            raise ValueError('Incompatible data and labels shapes.')
         data = np.atleast_3d(img_as_float(data))[..., np.newaxis]
     else:
         if data.ndim not in (3, 4):
             raise ValueError('For multichannel input, data must have 3 or 4 '
                              'dimensions.')
-        dims = data[..., 0].shape  # To reshape final labeled result
+        if data.shape[:-1] != labels.shape:
+            raise ValueError('Incompatible data and labels shapes.')
         data = img_as_float(data)
         if data.ndim == 3:  # 2D multispectral, needs singleton in 3rd axis
             data = data[:, :, np.newaxis, :]
 
-    # Spacing kwarg checks
-    if spacing is None:
-        spacing = np.asarray((1.,) * 3)
-    elif len(spacing) == len(dims):
-        if len(spacing) == 2:  # Need a dummy spacing for singleton 3rd dim
-            spacing = np.r_[spacing, 1.]
-        else:                  # Convert to array
-            spacing = np.asarray(spacing)
-    else:
-        raise ValueError('Input argument `spacing` incorrect, should be an '
-                         'iterable with one number per spatial dimension.')
+    labels_shape = labels.shape
+    labels_dtype = labels.dtype
 
     if copy:
         labels = np.copy(labels)
-    label_values = np.unique(labels)
 
-    # If some labeled pixels are isolated inside pruned zones, prune them
-    # as well and keep the labels for the final output
-    inds_isolated_seeds, isolated_values = _check_isolated_seeds(labels)
+    (labels, nlabels, mask,
+     inds_isolated_seeds, isolated_values) = _preprocess(labels)
 
-    # Reorder label values to have consecutive integers (no gaps)
-    if np.any(np.diff(label_values) != 1):
-        mask = labels >= 0
-        labels[mask] = rank_order(labels[mask])[0].astype(labels.dtype)
-    labels = labels.astype(np.int32)
+    if isolated_values is None:
+        # No non isolated zero valued areas in labels were
+        # found. Returning provided labels.
+        if return_full_prob:
+            # Return the concatenation of the masks of each unique label
+            return np.concatenate([np.atleast_3d(labels == lab)
+                                   for lab in np.unique(labels) if lab > 0],
+                                  axis=-1)
+        return labels
 
-    # If the array has pruned zones, be sure that no isolated pixels
-    # exist between pruned zones (they could not be determined)
-    if np.any(labels < 0):
-        filled = ndi.binary_propagation(labels > 0, mask=labels >= 0)
-        labels[np.logical_and(np.logical_not(filled), labels == 0)] = -1
-        del filled
-    labels = np.atleast_3d(labels)
+    # Build the linear system (lap_sparse, B)
+    lap_sparse, B = _build_linear_system(data, spacing, labels, nlabels, mask,
+                                         beta, multichannel)
 
-    # No unlabeled pixel, so nothing to do
-    if (labels == 0).sum() == 0:
-        labels = np.squeeze(labels)
-        labels[inds_isolated_seeds] = isolated_values
-        warn('Random walker only segments unlabeled areas, where '
-             'labels == 0. No zero valued areas in labels were '
-             'found. Returning provided labels.')
-        return _unchanged_labels(labels, return_full_prob)
-
-    if np.any(labels < 0):
-        lap_sparse = _build_laplacian(data, spacing, mask=labels >= 0,
-                                      beta=beta, multichannel=multichannel)
-    else:
-        lap_sparse = _build_laplacian(data, spacing, beta=beta,
-                                      multichannel=multichannel)
-    lap_sparse, B = _buildAB(lap_sparse, labels)
-
-    # We solve the linear system
-    # lap_sparse X = B
+    # Solve the linear system lap_sparse X = B
     # where X[i, j] is the probability that a marker of label i arrives
     # first at pixel j by anisotropic diffusion.
-    if mode == 'cg':
-        X = _solve_cg(lap_sparse, B, tol=tol,
-                      return_full_prob=return_full_prob)
-    if mode == 'cg_mg':
-        if not amg_loaded:
-            warn("""pyamg (http://pyamg.github.io/)) is needed to use
-                this mode, but is not installed. The 'cg' mode will be used
-                instead.""")
-            X = _solve_cg(lap_sparse, B, tol=tol,
-                          return_full_prob=return_full_prob)
-        else:
-            X = _solve_cg_mg(lap_sparse, B, tol=tol,
-                             return_full_prob=return_full_prob)
-    if mode == 'bf':
-        X = _solve_bf(lap_sparse, B,
-                      return_full_prob=return_full_prob)
+    X = _solve_linear_system(lap_sparse, B, tol, mode)
+
+    if X.min() < -prob_tol or X.max() > 1 + prob_tol:
+        warn('The probability range is outside [0, 1] given the tolerance '
+             '`prob_tol`. Consider decreasing `beta` and/or decreasing '
+             '`tol`.')
+
+    # Build the output according to return_full_prob value
+    # Put back labels of isolated seeds
+    labels[inds_isolated_seeds] = isolated_values
+    labels = labels.reshape(labels_shape)
+
+    mask = labels == 0
+    mask[inds_isolated_seeds] = False
 
-    # Clean up results
     if return_full_prob:
-        labels = labels.astype(np.float)
-        # Put back labels of isolated seeds
-        if len(isolated_values) > 0:
-            labels[inds_isolated_seeds] = isolated_values
-        X = np.array([_clean_labels_ar(Xline, labels, copy=True).reshape(dims)
-                      for Xline in X])
-        for i in range(1, int(labels.max()) + 1):
-            mask_i = np.squeeze(labels == i)
-            X[:, mask_i] = 0
-            X[i - 1, mask_i] = 1
+        out = np.zeros((nlabels,) + labels_shape)
+        for lab, (label_prob, prob) in enumerate(zip(out, X), start=1):
+            label_prob[mask] = prob
+            label_prob[labels == lab] = 1
     else:
-        X = _clean_labels_ar(X + 1, labels).reshape(dims)
-        # Put back labels of isolated seeds
-        X[inds_isolated_seeds] = isolated_values
-    return X
+        X = np.argmax(X, axis=0) + 1
+        out = labels.astype(labels_dtype)
+        out[mask] = X
 
-
-def _solve_bf(lap_sparse, B, return_full_prob=False):
-    """
-    solves lap_sparse X_i = B_i for each phase i. An LU decomposition
-    of lap_sparse is computed first. For each pixel, the label i
-    corresponding to the maximal X_i is returned.
-    """
-    lap_sparse = lap_sparse.tocsc()
-    solver = sparse.linalg.factorized(lap_sparse.astype(np.double))
-    X = np.array([solver(np.array((-B[i]).toarray()).ravel())
-                  for i in range(len(B))])
-    if not return_full_prob:
-        X = np.argmax(X, axis=0)
-    return X
-
-
-def _solve_cg(lap_sparse, B, tol, return_full_prob=False):
-    """
-    solves lap_sparse X_i = B_i for each phase i, using the conjugate
-    gradient method. For each pixel, the label i corresponding to the
-    maximal X_i is returned.
-    """
-    lap_sparse = lap_sparse.tocsc()
-    X = []
-    for i in range(len(B)):
-        x0 = cg(lap_sparse, -B[i].toarray(), tol=tol)[0]
-        X.append(x0)
-    if not return_full_prob:
-        X = np.array(X)
-        X = np.argmax(X, axis=0)
-    return X
-
-
-def _solve_cg_mg(lap_sparse, B, tol, return_full_prob=False):
-    """
-    solves lap_sparse X_i = B_i for each phase i, using the conjugate
-    gradient method with a multigrid preconditioner (ruge-stuben from
-    pyamg). For each pixel, the label i corresponding to the maximal
-    X_i is returned.
-    """
-    X = []
-    ml = ruge_stuben_solver(lap_sparse)
-    M = ml.aspreconditioner(cycle='V')
-    for i in range(len(B)):
-        x0 = cg(lap_sparse, -B[i].toarray(), tol=tol, M=M, maxiter=30)[0]
-        X.append(x0)
-    if not return_full_prob:
-        X = np.array(X)
-        X = np.argmax(X, axis=0)
-    return X
+    return out
