@@ -107,7 +107,7 @@ def test_hough_line_peaks():
 
 def test_hough_line_peaks_ordered():
     # Regression test per PR #1421
-    testim = np.zeros((256, 64), dtype=np.bool)
+    testim = np.zeros((256, 64), dtype=bool)
 
     testim[50:100, 20] = True
     testim[85:200, 25] = True
@@ -121,7 +121,7 @@ def test_hough_line_peaks_ordered():
 
 
 def test_hough_line_peaks_dist():
-    img = np.zeros((100, 100), dtype=np.bool_)
+    img = np.zeros((100, 100), dtype=bool)
     img[:, 30] = True
     img[:, 40] = True
     hspace, angles, dists = transform.hough_line(img)
@@ -136,7 +136,7 @@ def test_hough_line_peaks_angle():
 
 
 def check_hough_line_peaks_angle():
-    img = np.zeros((100, 100), dtype=np.bool_)
+    img = np.zeros((100, 100), dtype=bool)
     img[:, 0] = True
     img[0, :] = True
 
@@ -162,7 +162,7 @@ def check_hough_line_peaks_angle():
 
 
 def test_hough_line_peaks_num():
-    img = np.zeros((100, 100), dtype=np.bool_)
+    img = np.zeros((100, 100), dtype=bool)
     img[:, 30] = True
     img[:, 40] = True
     hspace, angles, dists = transform.hough_line(img)
@@ -178,8 +178,17 @@ def test_hough_line_peaks_zero_input():
     hspace, angles, dists = transform.hough_line(img, theta)
     h, a, d = transform.hough_line_peaks(hspace, angles, dists)
     assert_equal(a, np.array([]))
-    
-    
+
+
+def test_hough_line_peaks_single_angle():
+    # Regression test for gh-4814
+    # This code snippet used to raise an IndexError
+    img = np.random.random((100, 100))
+    tested_angles = np.array([np.pi / 2])
+    h, theta, d = transform.hough_line(img, theta=tested_angles)
+    accum, angles, dists = transform.hough_line_peaks(h, theta, d, threshold=2)
+
+
 @test_parallel()
 def test_hough_circle():
     # Prepare picture
@@ -260,6 +269,89 @@ def test_hough_circle_peaks_total_peak():
     assert_equal(out[3][0], np.array([rad_1, ]))
 
 
+def test_hough_circle_peaks_min_distance():
+    x_0, y_0, rad_0 = (50, 50, 20)
+    img = np.zeros((120, 100), dtype=int)
+    y, x = circle_perimeter(y_0, x_0, rad_0)
+    img[x, y] = 1
+
+    x_1, y_1, rad_1 = (60, 60, 30)
+    y, x = circle_perimeter(y_1, x_1, rad_1)
+    # Add noise and create an imperfect circle to lower the peak in Hough space
+    y[::2] += 1
+    x[::2] += 1
+    img[x, y] = 1
+
+    x_2, y_2, rad_2 = (70, 70, 20)
+    y, x = circle_perimeter(y_2, x_2, rad_2)
+    # Add noise and create an imperfect circle to lower the peak in Hough space
+    y[::2] += 1
+    x[::2] += 1
+    img[x, y] = 1
+
+    radii = [rad_0, rad_1, rad_2]
+    hspaces = transform.hough_circle(img, radii)
+    out = transform.hough_circle_peaks(hspaces, radii, min_xdistance=15,
+                                       min_ydistance=15, threshold=None,
+                                       num_peaks=np.inf,
+                                       total_num_peaks=np.inf,
+                                       normalize=True)
+
+    # The second circle is too close to the first one
+    # and has a weaker peak in Hough space due to imperfectness.
+    # Therefore it got removed.
+    assert_equal(out[1], np.array([y_0, y_2]))
+    assert_equal(out[2], np.array([x_0, x_2]))
+    assert_equal(out[3], np.array([rad_0, rad_2]))
+
+
+def test_hough_circle_peaks_total_peak_and_min_distance():
+    img = np.zeros((120, 120), dtype=int)
+    cx = cy = [40, 50, 60, 70, 80]
+    radii = range(20, 30, 2)
+    for i in range(len(cx)):
+        y, x = circle_perimeter(cy[i], cx[i], radii[i])
+        img[x, y] = 1
+
+    hspaces = transform.hough_circle(img, radii)
+    out = transform.hough_circle_peaks(hspaces, radii, min_xdistance=15,
+                                       min_ydistance=15, threshold=None,
+                                       num_peaks=np.inf,
+                                       total_num_peaks=2,
+                                       normalize=True)
+
+    # 2nd (4th) circle is removed as it is close to 1st (3rd) oneself.
+    # 5th is removed as total_num_peaks = 2
+    assert_equal(out[1], np.array(cy[:4:2]))
+    assert_equal(out[2], np.array(cx[:4:2]))
+    assert_equal(out[3], np.array(radii[:4:2]))
+
+
+def test_hough_circle_peaks_normalize():
+    x_0, y_0, rad_0 = (50, 50, 20)
+    img = np.zeros((120, 100), dtype=int)
+    y, x = circle_perimeter(y_0, x_0, rad_0)
+    img[x, y] = 1
+
+    x_1, y_1, rad_1 = (60, 60, 30)
+    y, x = circle_perimeter(y_1, x_1, rad_1)
+    img[x, y] = 1
+
+    radii = [rad_0, rad_1]
+    hspaces = transform.hough_circle(img, radii)
+    out = transform.hough_circle_peaks(hspaces, radii, min_xdistance=15,
+                                       min_ydistance=15, threshold=None,
+                                       num_peaks=np.inf,
+                                       total_num_peaks=np.inf,
+                                       normalize=False)
+
+    # Two perfect circles are close but the second one is bigger.
+    # Therefore, it is picked due to its high peak.
+    assert_equal(out[1], np.array([y_1]))
+    assert_equal(out[2], np.array([x_1]))
+    assert_equal(out[3], np.array([rad_1]))
+
+
 def test_hough_ellipse_zero_angle():
     img = np.zeros((25, 25), dtype=int)
     rx = 6
