@@ -1,20 +1,17 @@
-from __future__ import print_function, division
-
-import os
 import itertools
+import pytest
 
 import numpy as np
-from skimage import data_dir
-from skimage.io import imread
+from skimage.data import shepp_logan_phantom
 from skimage.transform import radon, iradon, iradon_sart, rescale
 
+from skimage._shared.utils import convert_to_float
 from skimage._shared import testing
 from skimage._shared.testing import test_parallel
 from skimage._shared._warnings import expected_warnings
 
 
-PHANTOM = imread(os.path.join(data_dir, "phantom.png"),
-                 as_gray=True)[::2, ::2]
+PHANTOM = shepp_logan_phantom()[::2, ::2]
 PHANTOM = rescale(PHANTOM, 0.5, order=1,
                   mode='constant', anti_aliasing=False, multichannel=False)
 
@@ -47,36 +44,54 @@ def _rescale_intensity(x):
     return x
 
 
-def check_radon_center(shape, circle):
+def test_iradon_bias_circular_phantom():
+    """
+    test that a uniform circular phantom has a small reconstruction bias
+    """
+    pixels = 128
+    xy = np.arange(-pixels / 2, pixels / 2) + 0.5
+    x, y = np.meshgrid(xy, xy)
+    image = x**2 + y**2 <= (pixels/4)**2
+
+    theta = np.linspace(0., 180., max(image.shape), endpoint=False)
+    sinogram = radon(image, theta=theta)
+
+    reconstruction_fbp = iradon(sinogram, theta=theta)
+    error = reconstruction_fbp - image
+
+    tol = 5e-5
+    roi_err = np.abs(np.mean(error))
+    assert roi_err < tol
+
+
+def check_radon_center(shape, circle, dtype, preserve_range):
     # Create a test image with only a single non-zero pixel at the origin
-    image = np.zeros(shape, dtype=np.float)
+    image = np.zeros(shape, dtype=dtype)
     image[(shape[0] // 2, shape[1] // 2)] = 1.
     # Calculate the sinogram
     theta = np.linspace(0., 180., max(shape), endpoint=False)
-    sinogram = radon(image, theta=theta, circle=circle)
+    sinogram = radon(image, theta=theta, circle=circle,
+                     preserve_range=preserve_range)
     # The sinogram should be a straight, horizontal line
     sinogram_max = np.argmax(sinogram, axis=0)
     print(sinogram_max)
     assert np.std(sinogram_max) < 1e-6
 
 
-shapes_for_test_radon_center = [(16, 16), (17, 17)]
-circles_for_test_radon_center = [False, True]
+@testing.parametrize("shape", [(16, 16), (17, 17)])
+@testing.parametrize("circle", [False, True])
+@testing.parametrize("dtype", [np.float64, np.float32, np.uint8, bool])
+@testing.parametrize("preserve_range", [False, True])
+def test_radon_center(shape, circle, dtype, preserve_range):
+    check_radon_center(shape, circle, dtype, preserve_range)
 
 
-@testing.parametrize("shape, circle",
-                     itertools.product(shapes_for_test_radon_center,
-                                       circles_for_test_radon_center))
-def test_radon_center(shape, circle):
-    check_radon_center(shape, circle)
-
-
-rectangular_shapes = [(32, 16), (33, 17)]
-
-
-@testing.parametrize("shape", rectangular_shapes)
-def test_radon_center_rectangular(shape):
-    check_radon_center(shape, False)
+@testing.parametrize("shape", [(32, 16), (33, 17)])
+@testing.parametrize("circle", [False])
+@testing.parametrize("dtype", [np.float64, np.float32, np.uint8, bool])
+@testing.parametrize("preserve_range", [False, True])
+def test_radon_center_rectangular(shape, circle, dtype, preserve_range):
+    check_radon_center(shape, circle, dtype, preserve_range)
 
 
 def check_iradon_center(size, theta, circle):
@@ -84,11 +99,11 @@ def check_iradon_center(size, theta, circle):
     # Create a test sinogram corresponding to a single projection
     # with a single non-zero pixel at the rotation center
     if circle:
-        sinogram = np.zeros((size, 1), dtype=np.float)
+        sinogram = np.zeros((size, 1), dtype=float)
         sinogram[size // 2, 0] = 1.
     else:
         diagonal = int(np.ceil(np.sqrt(2) * size))
-        sinogram = np.zeros((diagonal, 1), dtype=np.float)
+        sinogram = np.zeros((diagonal, 1), dtype=float)
         sinogram[sinogram.shape[0] // 2, 0] = 1.
     maxpoint = np.unravel_index(np.argmax(sinogram), sinogram.shape)
     print('shape of generated sinogram', sinogram.shape)
@@ -125,7 +140,7 @@ circles_for_test_iradon_center = [False, True]
 @testing.parametrize("size, theta, circle",
                      itertools.product(sizes_for_test_iradon_center,
                                        thetas_for_test_iradon_center,
-                                       circles_for_test_radon_center))
+                                       circles_for_test_iradon_center))
 def test_iradon_center(size, theta, circle):
     check_iradon_center(size, theta, circle)
 
@@ -133,7 +148,7 @@ def test_iradon_center(size, theta, circle):
 def check_radon_iradon(interpolation_type, filter_type):
     debug = False
     image = PHANTOM
-    reconstructed = iradon(radon(image, circle=False), filter=filter_type,
+    reconstructed = iradon(radon(image, circle=False), filter_name=filter_type,
                            interpolation=interpolation_type, circle=False)
     delta = np.mean(np.abs(image - reconstructed))
     print('\n\tmean error:', delta)
@@ -163,6 +178,15 @@ def test_radon_iradon(interpolation_type, filter_type):
     check_radon_iradon(interpolation_type, filter_type)
 
 
+@pytest.mark.parametrize("filter_type", filter_types)
+def test_iradon_new_signature(filter_type):
+    image = PHANTOM
+    sinogram = radon(image, circle=False)
+    with pytest.warns(FutureWarning):
+        assert np.array_equal(iradon(sinogram, filter=filter_type),
+                              iradon(sinogram, filter_name=filter_type))
+
+
 def test_iradon_angles():
     """
     Test with different number of projections
@@ -194,7 +218,7 @@ def test_iradon_angles():
 def check_radon_iradon_minimal(shape, slices):
     debug = False
     theta = np.arange(180)
-    image = np.zeros(shape, dtype=np.float)
+    image = np.zeros(shape, dtype=float)
     image[slices] = 1.
     sinogram = radon(image, theta, circle=False)
     reconstructed = iradon(sinogram, theta, circle=False)
@@ -398,9 +422,9 @@ def test_iradon_sart():
         np.random.seed(1239867)
         shifts = np.random.uniform(-3, 3, sinogram.shape[1])
         x = np.arange(sinogram.shape[0])
-        sinogram_shifted = np.vstack(np.interp(x + shifts[i], x,
-                                               sinogram[:, i])
-                                     for i in range(sinogram.shape[1])).T
+        sinogram_shifted = np.vstack([np.interp(x + shifts[i], x,
+                                                sinogram[:, i])
+                                      for i in range(sinogram.shape[1])]).T
         reconstructed = iradon_sart(sinogram_shifted, theta,
                                     projection_shifts=shifts)
         if debug:
@@ -419,3 +443,51 @@ def test_iradon_sart():
         delta = np.mean(np.abs(reconstructed - image))
         print('delta (1 iteration, shifted sinogram) =', delta)
         assert delta < 0.022 * error_factor
+
+
+@pytest.mark.parametrize("preserve_range", [True, False])
+def test_iradon_dtype(preserve_range):
+    sinogram = np.zeros((16, 1), dtype=int)
+    sinogram[8, 0] = 1.
+    sinogram64 = sinogram.astype('float64')
+    sinogram32 = sinogram.astype('float32')
+
+    assert iradon(sinogram, theta=[0],
+                  preserve_range=preserve_range).dtype == 'float64'
+    assert iradon(sinogram64, theta=[0],
+                  preserve_range=preserve_range).dtype == sinogram64.dtype
+    assert iradon(sinogram32, theta=[0],
+                  preserve_range=preserve_range).dtype == sinogram32.dtype
+
+
+def test_radon_dtype():
+    img = convert_to_float(PHANTOM, False)
+    img32 = img.astype(np.float32)
+
+    assert radon(img).dtype == img.dtype
+    assert radon(img32).dtype == img32.dtype
+
+
+@pytest.mark.parametrize("dtype", [np.float32, np.float64])
+def test_iradon_sart_dtype(dtype):
+    sinogram = np.zeros((16, 1), dtype=int)
+    sinogram[8, 0] = 1.
+    sinogram64 = sinogram.astype('float64')
+    sinogram32 = sinogram.astype('float32')
+
+    with expected_warnings(['Input data is cast to float']):
+        assert iradon_sart(sinogram, theta=[0]).dtype == 'float64'
+
+    assert iradon_sart(sinogram64, theta=[0]).dtype == sinogram64.dtype
+    assert iradon_sart(sinogram32, theta=[0]).dtype == sinogram32.dtype
+
+    assert iradon_sart(sinogram, theta=[0], dtype=dtype).dtype == dtype
+    assert iradon_sart(sinogram32, theta=[0], dtype=dtype).dtype == dtype
+    assert iradon_sart(sinogram64, theta=[0], dtype=dtype).dtype == dtype
+
+
+def test_iradon_sart_wrong_dtype():
+    sinogram = np.zeros((16, 1))
+
+    with testing.raises(ValueError):
+        iradon_sart(sinogram, dtype=int)
